# Copyright 2014 NetApp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock
from urllib import parse

import ddt
from oslo_db import exception as db_exception
from oslo_utils import timeutils
from webob import exc as webob_exc

from manila.api import common
from manila.api.openstack import api_version_request as api_version
from manila.api.v2 import share_networks
from manila.db import api as db_api
from manila import exception
from manila import quota
from manila.share import api as share_api
from manila import test
from manila.tests.api import fakes
from manila.tests import db_utils


fake_share_network_subnet = {
    'id': 'fake subnet id',
    'neutron_net_id': 'fake net id',
    'neutron_subnet_id': 'fake subnet id',
    'network_type': 'vlan',
    'segmentation_id': 1000,
    'cidr': '10.0.0.0/24',
    'ip_version': 4,
    'share_network_id': 'fake network id',
    'availability_zone_id': None,
    'share_servers': [],
    'availability_zone': []
}

fake_share_network = {
    'id': 'fake network id',
    'project_id': 'fake project',
    'created_at': timeutils.parse_strtime('2002-02-02', fmt="%Y-%m-%d"),
    'updated_at': None,
    'name': 'fake name',
    'description': 'fake description',
    'security_services': [],
    'share_network_subnets': [],
    'security_service_update_support': True,
    'status': 'active'
}


fake_share_network_shortened = {
    'id': 'fake network id',
    'name': 'fake name',
}

fake_share_network_with_ss = {
    'id': 'sn-id',
    'project_id': 'fake',
    'created_at': timeutils.parse_strtime('2001-01-01', fmt="%Y-%m-%d"),
    'updated_at': None,
    'name': 'test-sn',
    'description': 'fake description',
    'share_network_subnets': [],
    'security_services': [{'id': 'fake-ss-id'}]
}

fake_sn_with_ss_shortened = {
    'id': 'sn-id',
    'name': 'test-sn',
}

ADD_UPDATE_SEC_SERVICE_VERSION = '2.63'

QUOTAS = quota.QUOTAS


@ddt.ddt
class ShareNetworkAPITest(test.TestCase):

    def setUp(self):
        super(ShareNetworkAPITest, self).setUp()
        self.controller = share_networks.ShareNetworkController()
        self.req = fakes.HTTPRequest.blank('/share-networks')
        self.body = {share_networks.RESOURCE_NAME: {'name': 'fake name'}}
        self.context = self.req.environ['manila.context']
        self.share_api = share_api.API()

    def _check_share_network_view_shortened(self, view, share_nw):
        self.assertEqual(share_nw['id'], view['id'])
        self.assertEqual(share_nw['name'], view['name'])

    def _check_share_network_view(self, view, share_nw):
        self.assertEqual(share_nw['id'], view['id'])
        self.assertEqual(share_nw['project_id'], view['project_id'])
        self.assertEqual(share_nw['created_at'], view['created_at'])
        self.assertEqual(share_nw['updated_at'], view['updated_at'])
        self.assertEqual(share_nw['name'], view['name'])
        self.assertEqual(share_nw['description'], view['description'])
        self.assertNotIn('shares', view)
        self.assertNotIn('network_allocations', view)
        self.assertNotIn('security_services', view)

    def _setup_body_for_create_test(self, data):
        data.update({'user_id': 'fake_user_id'})
        body = {share_networks.RESOURCE_NAME: data}
        return body

    @ddt.data(
        {'neutron_net_id': 'fake', 'neutron_subnet_id': 'fake'})
    def test_create_valid_cases(self, data):
        body = self._setup_body_for_create_test(data)
        result = self.controller.create(self.req, body)
        data.pop('user_id', None)
        for k, v in data.items():
            self.assertIn(data[k], result['share_network'][k])

    @ddt.data(
        {'neutron_net_id': 'fake', 'neutron_subnet_id': 'fake',
         'availability_zone': 'fake'})
    def test_create_valid_cases_upper_2_50(self, data):
        req = fakes.HTTPRequest.blank('/share-networks', version="2.51")
        context = req.environ['manila.context']
        body = self._setup_body_for_create_test(data)
        fake_az = {
            'name': 'fake',
            'id': 'fake_id'
        }
        self.mock_object(db_api, 'availability_zone_get',
                         mock.Mock(return_value=fake_az))

        result = self.controller.create(req, body)
        result_subnet = result['share_network']['share_network_subnets'][0]
        data.pop('user_id', None)
        data.pop('project_id', None)
        data.pop('availability_zone_id', None)
        data.pop('id', None)
        data['availability_zone'] = result_subnet['availability_zone']

        for k, v in data.items():
            self.assertIn(k, result_subnet.keys())

        db_api.availability_zone_get.assert_called_once_with(
            context, fake_az['name']
        )

    @ddt.data(
        {'nova_net_id': 'foo', 'neutron_net_id': 'bar'},
        {'nova_net_id': 'foo', 'neutron_subnet_id': 'quuz'},
        {'nova_net_id': 'foo', 'neutron_net_id': 'bar',
         'neutron_subnet_id': 'quuz'},
        {'nova_net_id': 'fake_nova_net_id'},
        {'neutron_net_id': 'bar'},
        {'neutron_subnet_id': 'quuz'})
    def test_create_invalid_cases(self, data):
        data.update({'user_id': 'fake_user_id'})
        body = {share_networks.RESOURCE_NAME: data}
        self.assertRaises(
            webob_exc.HTTPBadRequest, self.controller.create, self.req, body)

    @ddt.data(
        {'name': 'new fake name'},
        {'description': 'new fake description'},
        {'name': 'new fake name', 'description': 'new fake description'})
    def test_update_valid_cases(self, data):
        body = {share_networks.RESOURCE_NAME: {'user_id': 'fake_user'}}
        created = self.controller.create(self.req, body)

        body = {share_networks.RESOURCE_NAME: data}
        result = self.controller.update(
            self.req, created['share_network']['id'], body)

        for k, v in data.items():
            self.assertIn(data[k], result['share_network'][k])

        self._check_share_network_view(
            result[share_networks.RESOURCE_NAME],
            result['share_network'])

    @ddt.data(
        {'nova_net_id': 'foo', 'neutron_net_id': 'bar'},
        {'nova_net_id': 'foo', 'neutron_subnet_id': 'quuz'},
        {'nova_net_id': 'foo', 'neutron_net_id': 'bar',
         'neutron_subnet_id': 'quuz'},
        {'nova_net_id': 'fake_nova_net_id'},
    )
    def test_update_invalid_cases(self, data):
        body = {share_networks.RESOURCE_NAME: {'user_id': 'fake_user'}}
        created = self.controller.create(self.req, body)
        body = {share_networks.RESOURCE_NAME: data}
        self.assertRaises(
            webob_exc.HTTPBadRequest,
            self.controller.update,
            self.req, created['share_network']['id'], body)

    @ddt.data(
        ({'share_network_subnets': [
            {'share_network_id': fake_share_network['id']}]}, True),
        ({'share_network_subnets': []}, False))
    @ddt.unpack
    def test__subnet_has_search_opt(self, network, has_search_opt):
        search_opts = {
            'share_network_id': fake_share_network['id']
        }

        result = None

        for key, value in search_opts.items():
            result = self.controller._subnet_has_search_opt(
                key, value, network)

        self.assertEqual(has_search_opt, result)

    def test_create_nominal(self):
        self.mock_object(db_api, 'share_network_subnet_create')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=fake_share_network))
        self.mock_object(common, 'check_net_id_and_subnet_id')
        with mock.patch.object(db_api,
                               'share_network_create',
                               mock.Mock(return_value=fake_share_network)):

            result = self.controller.create(self.req, self.body)

            db_api.share_network_create.assert_called_once_with(
                self.req.environ['manila.context'],
                self.body[share_networks.RESOURCE_NAME])

            self._check_share_network_view(
                result[share_networks.RESOURCE_NAME],
                fake_share_network)

    def test_create_db_api_exception(self):
        with mock.patch.object(db_api,
                               'share_network_create',
                               mock.Mock(side_effect=db_exception.DBError)):
            self.assertRaises(webob_exc.HTTPInternalServerError,
                              self.controller.create,
                              self.req,
                              self.body)

    def test_create_wrong_body(self):
        body = None
        self.assertRaises(webob_exc.HTTPUnprocessableEntity,
                          self.controller.create,
                          self.req,
                          body)

    @ddt.data(
        {'availability_zone': 'fake-zone'})
    def test_create_az_not_found(self, data):
        req = fakes.HTTPRequest.blank('/share-networks', version="2.51")

        self.mock_object(
            db_api, 'availability_zone_get',
            mock.Mock(
                side_effect=exception.AvailabilityZoneNotFound(id='fake')))

        body = {share_networks.RESOURCE_NAME: data}

        self.assertRaises(webob_exc.HTTPBadRequest,
                          self.controller.create,
                          req,
                          body)

    def test_create_error_on_subnet_creation(self):
        data = {
            'neutron_net_id': 'fake',
            'neutron_subnet_id': 'fake',
            'id': fake_share_network['id']
        }
        subnet_data = copy.deepcopy(data)
        self.mock_object(db_api, 'share_network_create',
                         mock.Mock(return_value=fake_share_network))
        self.mock_object(db_api, 'share_network_subnet_create',
                         mock.Mock(side_effect=db_exception.DBError()))
        self.mock_object(db_api, 'share_network_delete')
        body = {share_networks.RESOURCE_NAME: data}

        self.assertRaises(webob_exc.HTTPInternalServerError,
                          self.controller.create,
                          self.req,
                          body)

        db_api.share_network_create.assert_called_once_with(self.context, data)
        subnet_data['share_network_id'] = data['id']
        subnet_data.pop('id')
        db_api.share_network_subnet_create.assert_called_once_with(
            self.context, subnet_data)
        db_api.share_network_delete.assert_called_once_with(
            self.context, fake_share_network['id'])

    def test_delete_nominal(self):
        share_nw = fake_share_network.copy()
        subnet = fake_share_network_subnet.copy()
        subnet['share_servers'] = ['foo', 'bar']
        share_nw['share_network_subnets'] = [subnet]
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_nw))
        self.mock_object(db_api, 'share_instances_get_all_by_share_network',
                         mock.Mock(return_value=[]))
        self.mock_object(self.controller.share_rpcapi, 'delete_share_server')
        self.mock_object(self.controller,
                         '_all_share_servers_are_auto_deletable',
                         mock.Mock(return_value=True))
        self.mock_object(db_api, 'share_network_delete')

        self.controller.delete(self.req, share_nw['id'])

        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_nw['id'])
        (db_api.share_instances_get_all_by_share_network.
            assert_called_once_with(self.req.environ['manila.context'],
                                    share_nw['id']))
        self.controller.share_rpcapi.delete_share_server.assert_has_calls([
            mock.call(self.req.environ['manila.context'], 'foo'),
            mock.call(self.req.environ['manila.context'], 'bar')])
        db_api.share_network_delete.assert_called_once_with(
            self.req.environ['manila.context'], share_nw['id'])

    def test_delete_not_found(self):
        share_nw = 'fake network id'
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(side_effect=exception.ShareNetworkNotFound(
                             share_network_id=share_nw)))

        self.assertRaises(webob_exc.HTTPNotFound,
                          self.controller.delete,
                          self.req,
                          share_nw)

    def test_quota_delete_reservation_failed(self):
        share_nw = fake_share_network.copy()
        subnet = fake_share_network_subnet.copy()
        subnet['share_servers'] = ['foo', 'bar']
        share_nw['share_network_subnets'] = [subnet]
        share_nw['user_id'] = 'fake_user_id'

        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_nw))
        self.mock_object(db_api, 'share_instances_get_all_by_share_network',
                         mock.Mock(return_value=[]))
        self.mock_object(self.controller,
                         '_all_share_servers_are_auto_deletable',
                         mock.Mock(return_value=True))
        self.mock_object(self.controller.share_rpcapi, 'delete_share_server')
        self.mock_object(db_api, 'share_network_delete')
        self.mock_object(share_networks.QUOTAS, 'reserve',
                         mock.Mock(side_effect=Exception))
        self.mock_object(share_networks.QUOTAS, 'commit')

        self.controller.delete(self.req, share_nw['id'])

        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_nw['id'])

        (db_api.share_instances_get_all_by_share_network.
            assert_called_once_with(self.req.environ['manila.context'],
                                    share_nw['id']))

        self.controller.share_rpcapi.delete_share_server.assert_has_calls([
            mock.call(self.req.environ['manila.context'], 'foo'),
            mock.call(self.req.environ['manila.context'], 'bar')])
        db_api.share_network_delete.assert_called_once_with(
            self.req.environ['manila.context'], share_nw['id'])
        share_networks.QUOTAS.reserve.assert_called_once_with(
            self.req.environ['manila.context'],
            project_id=share_nw['project_id'],
            share_networks=-1,
            user_id=share_nw['user_id']
        )
        self.assertFalse(share_networks.QUOTAS.commit.called)

    def test_delete_in_use_by_share(self):
        share_nw = fake_share_network.copy()
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_nw))
        self.mock_object(db_api, 'share_instances_get_all_by_share_network',
                         mock.Mock(return_value=['foo', 'bar']))

        self.assertRaises(webob_exc.HTTPConflict,
                          self.controller.delete,
                          self.req,
                          share_nw['id'])

        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_nw['id'])
        (db_api.share_instances_get_all_by_share_network.
            assert_called_once_with(self.req.environ['manila.context'],
                                    share_nw['id']))

    def test_delete_in_use_by_share_group(self):
        share_nw = fake_share_network.copy()
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_nw))
        self.mock_object(db_api, 'count_share_groups_in_share_network',
                         mock.Mock(return_value=2))

        self.assertRaises(webob_exc.HTTPConflict,
                          self.controller.delete,
                          self.req,
                          share_nw['id'])

        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_nw['id'])

    def test_delete_contains_is_auto_deletable_false_servers(self):
        share_nw = fake_share_network.copy()
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_nw))
        self.mock_object(db_api, 'count_share_groups_in_share_network')
        self.mock_object(share_networks.ShareNetworkController,
                         '_all_share_servers_are_auto_deletable',
                         mock.Mock(return_value=False))

        self.assertRaises(webob_exc.HTTPConflict,
                          self.controller.delete,
                          self.req,
                          share_nw['id'])

        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_nw['id'])

    def test_delete_contains_more_than_one_subnet(self):
        share_nw = fake_share_network.copy()
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_nw))
        self.mock_object(db_api, 'share_instances_get_all_by_share_network',
                         mock.Mock(return_value=None))
        self.mock_object(db_api, 'count_share_groups_in_share_network',
                         mock.Mock(return_value=None))
        self.mock_object(self.controller, '_share_network_contains_subnets',
                         mock.Mock(return_value=True))

        self.assertRaises(webob_exc.HTTPConflict,
                          self.controller.delete,
                          self.req,
                          share_nw['id'])

        db_api.share_network_get.assert_called_once_with(
            self.context, share_nw['id'])
        (db_api.share_instances_get_all_by_share_network
            .assert_called_once_with(self.context, share_nw['id']))
        db_api.count_share_groups_in_share_network.assert_called_once_with(
            self.context, share_nw['id']
        )
        (self.controller._share_network_contains_subnets
            .assert_called_once_with(share_nw))

    def test_delete_subnet_contains_share_server(self):
        share_nw = fake_share_network.copy()
        share_nw['share_network_subnets'].append({
            'id': 'fake_sns_id',
            'share_servers': [{'id': 'fake_share_server_id'}]
        })
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_nw))
        self.mock_object(db_api, 'count_share_groups_in_share_network',
                         mock.Mock(return_value=0))
        self.mock_object(self.controller, '_share_network_contains_subnets',
                         mock.Mock(return_value=False))
        self.mock_object(
            self.controller, '_all_share_servers_are_auto_deletable',
            mock.Mock(return_value=False))

        self.assertRaises(webob_exc.HTTPConflict,
                          self.controller.delete,
                          self.req,
                          share_nw['id'])

    @ddt.data(
        ({'share_servers': [{'is_auto_deletable': True},
                            {'is_auto_deletable': True}]}, True),
        ({'share_servers': [{'is_auto_deletable': True},
                            {'is_auto_deletable': False}]}, False),
    )
    @ddt.unpack
    def test__share_servers_are_auto_deletable(self, fake_share_network,
                                               expected_result):
        self.assertEqual(
            expected_result,
            self.controller._all_share_servers_are_auto_deletable(
                fake_share_network))

    @ddt.data(
        ({'share_network_subnets': [{'share_servers': [{}, {}]}]}, True),
        ({'share_network_subnets': [{'share_servers': []}]}, False),
    )
    @ddt.unpack
    def test__share_network_subnets_contain_share_servers(self, share_network,
                                                          expected_result):
        self.assertEqual(
            expected_result,
            self.controller._share_network_subnets_contain_share_servers(
                share_network))

    def test_show_nominal(self):
        share_nw = 'fake network id'
        with mock.patch.object(db_api,
                               'share_network_get',
                               mock.Mock(return_value=fake_share_network)):
            result = self.controller.show(self.req, share_nw)

            db_api.share_network_get.assert_called_once_with(
                self.req.environ['manila.context'],
                share_nw)

            self._check_share_network_view(
                result[share_networks.RESOURCE_NAME],
                fake_share_network)

    def test_show_not_found(self):
        share_nw = 'fake network id'
        test_exception = exception.ShareNetworkNotFound(
            share_network_id=share_nw)
        with mock.patch.object(db_api,
                               'share_network_get',
                               mock.Mock(side_effect=test_exception)):
            self.assertRaises(webob_exc.HTTPNotFound,
                              self.controller.show,
                              self.req,
                              share_nw)

    def test_index_no_filters(self):
        networks = [fake_share_network]
        with mock.patch.object(db_api,
                               'share_network_get_all_by_filter',
                               mock.Mock(return_value=networks)):

            result = self.controller.index(self.req)

            db_api.share_network_get_all_by_filter.assert_called_once_with(
                self.context, filters={})

            self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
            self._check_share_network_view_shortened(
                result[share_networks.RESOURCES_NAME][0],
                fake_share_network_shortened)

    def test_index_detailed(self):
        networks = [fake_share_network]
        with mock.patch.object(db_api,
                               'share_network_get_all_by_filter',
                               mock.Mock(return_value=networks)):

            result = self.controller.detail(self.req)

            db_api.share_network_get_all_by_filter.assert_called_once_with(
                self.context, filters={})

            self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
            self._check_share_network_view(
                result[share_networks.RESOURCES_NAME][0],
                fake_share_network)

    @mock.patch.object(db_api, 'share_network_get_all_by_filter',
                       mock.Mock())
    def test_index_filter_by_security_service(self):
        db_api.share_network_get_all_by_filter.return_value = [
            fake_share_network_with_ss]
        req = fakes.HTTPRequest.blank(
            '/share_networks?security_service_id=fake-ss-id')
        result = self.controller.index(req)
        filters = {'security_service_id': 'fake-ss-id'}
        (db_api.share_network_get_all_by_filter.
            assert_called_once_with(req.environ['manila.context'],
                                    filters=filters))
        self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
        self._check_share_network_view_shortened(
            result[share_networks.RESOURCES_NAME][0],
            fake_sn_with_ss_shortened)

    @mock.patch.object(db_api, 'share_network_get_all_by_filter', mock.Mock())
    def test_index_all_tenants_non_admin_context(self):
        req = fakes.HTTPRequest.blank(
            '/share_networks?all_tenants=1')
        fake_context = req.environ['manila.context']
        db_api.share_network_get_all_by_filter.return_value = []
        self.controller.index(req)
        db_api.share_network_get_all_by_filter.assert_called_with(
            fake_context, filters={})

    @mock.patch.object(db_api, 'share_network_get_all_by_filter', mock.Mock())
    def test_index_all_tenants_admin_context(self):
        db_api.share_network_get_all_by_filter.return_value = [
            fake_share_network]
        req = fakes.HTTPRequest.blank(
            '/share_networks?all_tenants=1',
            use_admin_context=True)
        result = self.controller.index(req)
        db_api.share_network_get_all_by_filter.assert_called_once_with(
            req.environ['manila.context'], filters={})
        self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
        self._check_share_network_view_shortened(
            result[share_networks.RESOURCES_NAME][0],
            fake_share_network_shortened)

    @mock.patch.object(db_api, 'share_network_get_all', mock.Mock())
    def test_index_all_tenants_with_invaild_value(self):
        req = fakes.HTTPRequest.blank(
            '/share_networks?all_tenants=wonk',
            use_admin_context=True)

        self.assertRaises(exception.InvalidInput, self.controller.index, req)

    @mock.patch.object(db_api, 'share_network_get_all_by_filter', mock.Mock())
    def test_index_all_tenants_with_value_zero(self):
        db_api.share_network_get_all_by_filter.return_value = [
            fake_share_network]
        req = fakes.HTTPRequest.blank(
            '/share_networks?all_tenants=0',
            use_admin_context=True)

        result = self.controller.index(req)

        self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
        self._check_share_network_view_shortened(
            result[share_networks.RESOURCES_NAME][0],
            fake_share_network_shortened)
        filters = {'project_id': 'fake'}
        db_api.share_network_get_all_by_filter.assert_called_once_with(
            req.environ['manila.context'], filters=filters)

    @mock.patch.object(db_api, 'share_network_get_all_by_filter', mock.Mock())
    def test_index_filter_by_project_id_non_admin_context(self):
        req = fakes.HTTPRequest.blank(
            '/share_networks?project_id=fake project')
        fake_context = req.environ['manila.context']
        db_api.share_network_get_all_by_filter.return_value = []
        self.controller.index(req)
        db_api.share_network_get_all_by_filter.assert_called_with(
            fake_context, filters={})

    @mock.patch.object(db_api, 'share_network_get_all_by_filter', mock.Mock())
    def test_index_filter_by_project_id_admin_context(self):
        db_api.share_network_get_all_by_filter.return_value = [
            fake_share_network_with_ss
        ]
        req = fakes.HTTPRequest.blank(
            '/share_networks?project_id=fake',
            use_admin_context=True)
        result = self.controller.index(req)
        filters = {'project_id': 'fake'}
        db_api.share_network_get_all_by_filter.assert_called_once_with(
            req.environ['manila.context'], filters=filters)
        self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
        self._check_share_network_view_shortened(
            result[share_networks.RESOURCES_NAME][0],
            fake_sn_with_ss_shortened)

    @mock.patch.object(db_api, 'share_network_get_all_by_filter',
                       mock.Mock())
    def test_index_filter_by_ss_and_project_id_admin_context(self):
        db_api.share_network_get_all_by_filter.return_value = [
            fake_share_network_with_ss
        ]
        req = fakes.HTTPRequest.blank(
            '/share_networks?security_service_id=fake-ss-id&project_id=fake',
            use_admin_context=True)
        result = self.controller.index(req)
        filters = {'project_id': 'fake',
                   'security_service_id': 'fake-ss-id'}
        db_api.share_network_get_all_by_filter.assert_called_once_with(
            req.environ['manila.context'], filters=filters)
        self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
        self._check_share_network_view_shortened(
            result[share_networks.RESOURCES_NAME][0],
            fake_sn_with_ss_shortened)

    @ddt.data(('name=fo', 0), ('description=d', 0),
              ('name=foo&description=d', 0),
              ('name=foo', 1), ('description=ds', 1),
              ('name~=foo&description~=ds', 2),
              ('name=foo&description~=ds', 1),
              ('name~=foo&description=ds', 1))
    @ddt.unpack
    @mock.patch.object(db_api, 'share_network_get_all_by_filter',
                       mock.Mock())
    def test_index_filter_by_name_and_description(
            self, filter, share_network_number):
        fake_objs = [{'name': 'fo2', 'description': 'd2', 'id': 'fake1'},
                     {'name': 'foo', 'description': 'ds', 'id': 'fake2'},
                     {'name': 'foo1', 'description': 'ds1', 'id': 'fake3'}]
        db_api.share_network_get_all_by_filter.return_value = fake_objs
        req = fakes.HTTPRequest.blank(
            '/share_networks?' + filter,
            use_admin_context=True, version='2.36')
        result = self.controller.index(req)
        filters = {'project_id': self.context.project_id}
        db_api.share_network_get_all_by_filter.assert_called_with(
            req.environ['manila.context'], filters=filters)
        self.assertEqual(share_network_number,
                         len(result[share_networks.RESOURCES_NAME]))
        if share_network_number > 0:
            self._check_share_network_view_shortened(
                result[share_networks.RESOURCES_NAME][0], fake_objs[1])
        if share_network_number > 1:
            self._check_share_network_view_shortened(
                result[share_networks.RESOURCES_NAME][1], fake_objs[2])

    @mock.patch.object(db_api, 'share_network_get_all_by_filter',
                       mock.Mock())
    def test_index_all_filter_opts(self):
        valid_filter_opts = {
            'created_before': '2001-02-02',
            'created_since': '1999-01-01',
            'name': 'test-sn'
        }
        db_api.share_network_get_all_by_filter.return_value = [
            fake_share_network_with_ss]

        query_string = '/share-networks?' + parse.urlencode(sorted(
            [(k, v) for (k, v) in list(valid_filter_opts.items())]))
        for use_admin_context in [True, False]:
            req = fakes.HTTPRequest.blank(query_string,
                                          use_admin_context=use_admin_context)
            result = self.controller.index(req)
            parsed_created_before = timeutils.parse_strtime(
                valid_filter_opts['created_before'], fmt="%Y-%m-%d")
            parsed_created_since = timeutils.parse_strtime(
                valid_filter_opts['created_since'], fmt="%Y-%m-%d")
            filters = {'created_before': parsed_created_before,
                       'created_since': parsed_created_since}
            if use_admin_context:
                filters['project_id'] = 'fake'
            db_api.share_network_get_all_by_filter.assert_called_with(
                req.environ['manila.context'], filters=filters)
            self.assertEqual(1, len(result[share_networks.RESOURCES_NAME]))
            self._check_share_network_view_shortened(
                result[share_networks.RESOURCES_NAME][0],
                fake_sn_with_ss_shortened)

    @mock.patch.object(db_api, 'share_network_get', mock.Mock())
    def test_update_nominal(self):
        share_nw = 'fake network id'
        db_api.share_network_get.return_value = fake_share_network

        body = {share_networks.RESOURCE_NAME: {'name': 'new name'}}

        with mock.patch.object(db_api,
                               'share_network_update',
                               mock.Mock(return_value=fake_share_network)):
            result = self.controller.update(self.req, share_nw, body)

            db_api.share_network_update.assert_called_once_with(
                self.req.environ['manila.context'],
                share_nw,
                body[share_networks.RESOURCE_NAME])

            self._check_share_network_view(
                result[share_networks.RESOURCE_NAME],
                fake_share_network)

    @mock.patch.object(db_api, 'share_network_get', mock.Mock())
    def test_update_not_found(self):
        share_nw = 'fake network id'
        db_api.share_network_get.side_effect = exception.ShareNetworkNotFound(
            share_network_id=share_nw)

        self.assertRaises(webob_exc.HTTPNotFound,
                          self.controller.update,
                          self.req,
                          share_nw,
                          self.body)

    @mock.patch.object(db_api, 'share_network_get', mock.Mock())
    def test_update_invalid_key_in_use(self):
        share_nw = fake_share_network.copy()
        subnet = fake_share_network_subnet.copy()
        subnet['share_servers'] = [{'id': 1}]
        share_nw['share_network_subnets'] = [subnet]

        db_api.share_network_get.return_value = share_nw
        body = {
            share_networks.RESOURCE_NAME: {
                'name': 'new name',
                'user_id': 'new id',
            },
        }
        self.assertRaises(webob_exc.HTTPForbidden,
                          self.controller.update,
                          self.req,
                          share_nw['id'],
                          body)

    @mock.patch.object(db_api, 'share_network_get', mock.Mock())
    @mock.patch.object(db_api, 'share_network_update', mock.Mock())
    def test_update_valid_keys_in_use(self):
        share_nw = fake_share_network.copy()
        subnet = fake_share_network_subnet.copy()
        subnet['share_servers'] = [{'id': 1}]
        share_nw['share_network_subnets'] = [subnet]
        updated_share_nw = share_nw.copy()
        updated_share_nw['name'] = 'new name'
        updated_share_nw['description'] = 'new description'

        db_api.share_network_get.return_value = share_nw
        db_api.share_network_update.return_value = updated_share_nw
        body = {
            share_networks.RESOURCE_NAME: {
                'name': updated_share_nw['name'],
                'description': updated_share_nw['description'],
            },
        }
        self.controller.update(self.req, share_nw['id'], body)
        db_api.share_network_get.assert_called_once_with(self.context,
                                                         share_nw['id'])
        db_api.share_network_update.assert_called_once_with(
            self.context, share_nw['id'], body['share_network'])

    @mock.patch.object(db_api, 'share_network_get', mock.Mock())
    def test_update_db_api_exception(self):
        share_nw = 'fake network id'
        db_api.share_network_get.return_value = fake_share_network

        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=False))
        self.mock_object(db_api, 'share_network_subnet_get_default_subnet',
                         mock.Mock(return_value=fake_share_network_subnet))
        self.mock_object(db_api, 'share_network_subnet_update')

        body = {share_networks.RESOURCE_NAME: {'neutron_subnet_id':
                                               'new subnet'}}

        with mock.patch.object(db_api,
                               'share_network_update',
                               mock.Mock(side_effect=db_exception.DBError)):
            self.assertRaises(webob_exc.HTTPBadRequest,
                              self.controller.update,
                              self.req,
                              share_nw,
                              body)
        db_api.share_network_subnet_get_default_subnet.assert_called_once_with(
            self.context, share_nw)
        db_api.share_network_subnet_update.assert_called_once_with(
            self.context, fake_share_network_subnet['id'],
            body['share_network'])

    @ddt.data((webob_exc.HTTPBadRequest, fake_share_network_subnet, None,
               'new subnet'),
              (webob_exc.HTTPBadRequest, None, 'neutron net', None))
    @ddt.unpack
    def test_update_default_subnet_errors(self, exception_to_raise,
                                          get_default_subnet_return,
                                          neutron_net_id, neutron_subnet_id):
        share_nw = 'fake network id'
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=fake_share_network))
        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=False))
        self.mock_object(db_api, 'share_network_subnet_get_default_subnet',
                         mock.Mock(return_value=get_default_subnet_return))

        if get_default_subnet_return:
            fake_subnet = copy.deepcopy(fake_share_network_subnet)
            fake_subnet['neutron_net_id'] = None
            fake_subnet['neutron_subnet_id'] = None
            db_api.share_network_subnet_get_default_subnet.return_value = (
                fake_subnet)
        body = {
            share_networks.RESOURCE_NAME: {
                'neutron_net_id': neutron_net_id,
                'neutron_subnet_id': neutron_subnet_id
            }
        }

        self.assertRaises(exception_to_raise,
                          self.controller.update,
                          self.req,
                          share_nw,
                          body)

        db_api.share_network_subnet_get_default_subnet.assert_called_once_with(
            self.context, share_nw)

    @ddt.data(*set(("1.0", "2.25", "2.26", api_version._MAX_API_VERSION)))
    def test_add_security_service(self, microversion):
        share_network_id = 'fake network id'
        security_service_id = 'fake ss id'
        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers')

        body = {'add_security_service': {'security_service_id':
                                         security_service_id}}

        req = fakes.HTTPRequest.blank('/share-networks', version=microversion)
        with mock.patch.object(self.controller, 'add_security_service',
                               mock.Mock()):
            self.controller.add_security_service(req, share_network_id, body)
            self.controller.add_security_service.assert_called_once_with(
                req, share_network_id, body)

    def _setup_add_sec_services_with_servers_tests(
            self, share_network, security_service, network_is_active=True,
            version=ADD_UPDATE_SEC_SERVICE_VERSION,
            share_api_update_services_action=mock.Mock()):
        self.mock_object(
            db_api, 'share_network_get', mock.Mock(return_value=share_network))
        self.mock_object(
            db_api, 'security_service_get',
            mock.Mock(return_value=security_service))
        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=True))
        self.mock_object(
            self.controller.share_api, 'update_share_network_security_service',
            share_api_update_services_action)
        self.mock_object(
            common, 'check_share_network_is_active',
            mock.Mock(return_value=network_is_active))
        self.mock_object(db_api, 'share_network_add_security_service')
        self.mock_object(self.controller._view_builder, 'build_share_network')

        body = {
            'add_security_service': {
                'security_service_id': security_service['id']
            }
        }
        req = fakes.HTTPRequest.blank(
            '/add_security_service', version=version, use_admin_context=True)
        context = req.environ['manila.context']

        return req, context, body

    def test_add_security_service_with_servers(self):
        security_service = db_utils.create_security_service()
        security_service_id = security_service['id']
        share_network = db_utils.create_share_network()
        share_network_id = share_network['id']
        req, context, body = self._setup_add_sec_services_with_servers_tests(
            share_network, security_service)

        self.controller.add_security_service(req, share_network_id, body)

        db_api.security_service_get.assert_called_once_with(
            context, security_service_id)
        (self.controller._share_network_subnets_contain_share_servers.
         assert_called_once_with(share_network))
        db_api.share_network_get.assert_called_once_with(
            context, share_network_id)
        (self.controller.share_api.update_share_network_security_service.
         assert_called_once_with(context, share_network, security_service))

    def test_add_security_service_with_server_invalid_version(self):
        security_service = db_utils.create_security_service()
        security_service_id = security_service['id']
        share_network = db_utils.create_share_network()
        share_network_id = share_network['id']
        req, context, body = self._setup_add_sec_services_with_servers_tests(
            share_network, security_service, version='2.59')

        self.assertRaises(
            webob_exc.HTTPForbidden,
            self.controller.add_security_service,
            req, share_network_id, body
        )

        db_api.security_service_get.assert_called_once_with(
            context, security_service_id)
        (self.controller._share_network_subnets_contain_share_servers.
         assert_called_once_with(share_network))
        db_api.share_network_get.assert_called_once_with(
            context, share_network_id)

    @ddt.data(
        (exception.ServiceIsDown(message='fake'), webob_exc.HTTPConflict),
        (exception.InvalidShareNetwork(message='fake'),
         webob_exc.HTTPBadRequest)
    )
    @ddt.unpack
    def test_add_security_service_with_server_update_failed(
            self, side_effect, exception_to_raise):
        security_service = db_utils.create_security_service()
        security_service_id = security_service['id']
        share_network_id = fake_share_network['id']
        fake_share_network['security_service_update_support'] = True
        action = mock.Mock(side_effect=side_effect)

        req, context, body = self._setup_add_sec_services_with_servers_tests(
            fake_share_network, security_service,
            share_api_update_services_action=action)

        self.assertRaises(
            exception_to_raise,
            self.controller.add_security_service,
            req, share_network_id, body
        )

        db_api.security_service_get.assert_called_once_with(
            context, security_service_id)
        db_api.share_network_get.assert_called_once_with(
            context, share_network_id)
        (self.controller.share_api.update_share_network_security_service.
         assert_called_once_with(context, fake_share_network,
                                 security_service))

    @ddt.data(
        (exception.NotFound(message='fake'), webob_exc.HTTPNotFound),
        (exception.ShareNetworkSecurityServiceAssociationError(message='fake'),
         webob_exc.HTTPBadRequest))
    @ddt.unpack
    def test_action_add_security_service_conflict(self, captured_exception,
                                                  expected_raised_exception):
        share_network = fake_share_network.copy()
        share_network['security_services'] = [{'id': 'security_service_1',
                                               'type': 'ldap'}]
        security_service = {'id': ' security_service_2',
                            'type': 'ldap'}
        body = {'add_security_service': {'security_service_id':
                                         security_service['id']}}
        request = fakes.HTTPRequest.blank(
            '/share-networks', use_admin_context=True)
        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=False))
        update_sec_serv_mock = self.mock_object(
            self.controller.share_api, 'update_share_network_security_service')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(db_api, 'security_service_get',
                         mock.Mock(return_value=security_service))
        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(
            db_api, 'share_network_add_security_service',
            mock.Mock(side_effect=captured_exception))

        db_api.security_service_get.return_value = security_service
        db_api.share_network_get.return_value = share_network
        self.assertRaises(expected_raised_exception,
                          self.controller.add_security_service,
                          request,
                          share_network['id'],
                          body)

        db_api.share_network_get.assert_called_once_with(
            request.environ['manila.context'], share_network['id'])
        db_api.security_service_get.assert_called_once_with(
            request.environ['manila.context'], security_service['id'])
        share_networks.policy.check_policy.assert_called_once_with(
            request.environ['manila.context'],
            share_networks.RESOURCE_NAME,
            'add_security_service', target_obj=share_network)
        update_sec_serv_mock.assert_called_once_with(
            request.environ['manila.context'], share_network,
            security_service)

    def _setup_update_sec_services_with_servers_tests(
            self, share_network, security_services,
            version=ADD_UPDATE_SEC_SERVICE_VERSION,
            share_api_update_services_action=mock.Mock()):

        self.mock_object(
            db_api, 'share_network_get', mock.Mock(return_value=share_network))
        self.mock_object(
            db_api, 'security_service_get',
            mock.Mock(side_effect=security_services))
        self.mock_object(
            self.controller.share_api, 'update_share_network_security_service',
            share_api_update_services_action)
        self.mock_object(self.controller._view_builder, 'build_share_network')
        self.mock_object(db_api, 'share_network_update_security_service')

        body = {
            'update_security_service': {
                'current_service_id': security_services[0]['id'],
                'new_service_id': security_services[1]['id']
            }
        }
        req = fakes.HTTPRequest.blank(
            '/add_security_service', version=version, use_admin_context=True)
        context = req.environ['manila.context']

        return req, context, body

    def test_update_security_service_service_not_found(self):
        security_services = [
            db_utils.create_security_service() for i in range(2)]
        share_network = copy.deepcopy(fake_share_network)
        share_network['security_service_update_support'] = True

        req, context, body = (
            self._setup_update_sec_services_with_servers_tests(
                share_network, security_services))

        db_api.security_service_get.side_effect = exception.NotFound('fake')

        self.assertRaises(
            webob_exc.HTTPBadRequest,
            self.controller.update_security_service,
            req, share_network['id'], body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_has_calls(
            [mock.call(context, security_services[0]['id'])]
        )

    @ddt.data(
        (exception.ServiceIsDown(message='fake'), webob_exc.HTTPConflict),
        (exception.InvalidShareNetwork(message='fake'),
         webob_exc.HTTPBadRequest))
    @ddt.unpack
    def test_update_security_service_share_api_failure(self, side_effect, exc):
        security_services = [
            db_utils.create_security_service() for i in range(2)]
        share_network = copy.deepcopy(fake_share_network)
        share_network['security_service_update_support'] = True

        req, context, body = (
            self._setup_update_sec_services_with_servers_tests(
                share_network, security_services,
                share_api_update_services_action=mock.Mock(
                    side_effect=side_effect)))

        self.assertRaises(
            exc,
            self.controller.update_security_service,
            req, share_network['id'], body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_has_calls(
            [mock.call(context, security_services[0]['id']),
             mock.call(context, security_services[1]['id'])]
        )

    def test_update_security_service(self):
        security_services = [
            db_utils.create_security_service() for i in range(2)]
        share_network = copy.copy(fake_share_network)
        share_network['security_service_update_support'] = True

        req, context, body = (
            self._setup_update_sec_services_with_servers_tests(
                share_network, security_services))

        self.controller.update_security_service(
            req, share_network['id'], body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_has_calls(
            [mock.call(context, security_services[0]['id']),
             mock.call(context, security_services[1]['id'])]
        )
        (self.controller.share_api.update_share_network_security_service.
            assert_called_once_with(
                context, share_network, security_services[1],
                current_security_service=security_services[0]))
        db_api.share_network_update_security_service.assert_called_once_with(
            context, share_network['id'], security_services[0]['id'],
            security_services[1]['id'])

    @ddt.data(*set(("1.0", "2.25", "2.26", api_version._MAX_API_VERSION)))
    def test_action_remove_security_service(self, microversion):
        share_network_id = 'fake network id'
        security_service_id = 'fake ss id'
        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers')
        body = {'remove_security_service': {'security_service_id':
                                            security_service_id}}

        req = fakes.HTTPRequest.blank('/share-networks', version=microversion)
        with mock.patch.object(self.controller, 'remove_security_service',
                               mock.Mock()):
            self.controller.remove_security_service(
                req, share_network_id, body)
            self.controller.remove_security_service.assert_called_once_with(
                req, share_network_id, body)

    @mock.patch.object(db_api, 'share_network_get', mock.Mock())
    @mock.patch.object(share_networks.policy, 'check_policy', mock.Mock())
    def test_action_remove_security_service_forbidden(self):
        share_network = fake_share_network.copy()
        subnet = fake_share_network_subnet.copy()
        subnet['share_servers'] = ['foo']
        share_network['share_network_subnets'] = [subnet]
        db_api.share_network_get.return_value = share_network
        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=True))
        body = {
            'remove_security_service': {
                'security_service_id': 'fake id',
            },
        }
        self.assertRaises(webob_exc.HTTPForbidden,
                          self.controller.remove_security_service,
                          self.req,
                          share_network['id'],
                          body)
        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_network['id'])
        share_networks.policy.check_policy.assert_called_once_with(
            self.req.environ['manila.context'],
            share_networks.RESOURCE_NAME,
            'remove_security_service', target_obj=share_network)

    @mock.patch.object(share_networks.policy, 'check_policy', mock.Mock())
    def test_action_remove_security_service_share_no_share_servers(self):
        share_network = fake_share_network.copy()
        subnet = fake_share_network_subnet.copy()
        share_network['share_network_subnets'] = [subnet]
        mock_share_net_get = self.mock_object(
            db_api, 'share_network_get', mock.Mock(return_value=share_network))
        mock_contains_servers = self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=False))
        mock_api_remove = self.mock_object(
            db_api, 'share_network_remove_security_service',
            mock.Mock(return_value=share_network))
        mock_view = self.mock_object(
            self.controller._view_builder, 'build_share_network',
            mock.Mock(return_value='fake_view'))
        body = {
            'remove_security_service': {
                'security_service_id': 'fake_ss_id',
            },
        }

        view = self.controller.remove_security_service(
            self.req, share_network['id'], body)

        self.assertEqual('fake_view', view)
        mock_share_net_get.assert_called_once_with(
            self.req.environ['manila.context'], share_network['id'])
        share_networks.policy.check_policy.assert_called_once_with(
            self.req.environ['manila.context'],
            share_networks.RESOURCE_NAME,
            'remove_security_service', target_obj=share_network)
        mock_contains_servers.assert_called_once_with(share_network)
        mock_api_remove.assert_called_once_with(
            self.req.environ['manila.context'], share_network['id'],
            'fake_ss_id')
        mock_view.assert_called_once_with(self.req, share_network)

    @ddt.data(
        (KeyError, webob_exc.HTTPBadRequest),
        (exception.NotFound(message='fake'), webob_exc.HTTPNotFound),
        (exception.ShareNetworkSecurityServiceDissociationError(
            message='fake'), webob_exc.HTTPBadRequest))
    @ddt.unpack
    def test_action_remove_security_service_share_api_exception(self, api_exp,
                                                                expect_exp):
        share_network = fake_share_network.copy()
        subnet = fake_share_network_subnet.copy()
        share_network['share_network_subnets'] = [subnet]
        mock_policy = self.mock_object(
            share_networks.policy, 'check_policy', mock.Mock())
        mock_share_net_get = self.mock_object(
            db_api, 'share_network_get',
            mock.Mock(return_value=share_network))
        mock_contains_servers = self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=False))
        mock_api_remove = self.mock_object(
            db_api, 'share_network_remove_security_service',
            mock.Mock(side_effect=api_exp))
        body = {
            'remove_security_service': {
                'security_service_id': 'fake_ss_id',
            },
        }

        self.assertRaises(expect_exp,
                          self.controller.remove_security_service,
                          self.req,
                          share_network['id'],
                          body)

        mock_share_net_get.assert_called_once_with(
            self.req.environ['manila.context'], share_network['id'])
        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_network['id'])
        mock_policy.assert_called_once_with(
            self.req.environ['manila.context'],
            share_networks.RESOURCE_NAME,
            'remove_security_service', target_obj=share_network)
        mock_contains_servers.assert_called_once_with(share_network)
        mock_api_remove.assert_called_once_with(
            self.req.environ['manila.context'], share_network['id'],
            'fake_ss_id')

    @ddt.data('add_security_service', 'remove_security_service')
    def test_action_security_service_contains_share_servers(self, action):
        share_network = fake_share_network.copy()
        security_service = {'id': ' security_service_2',
                            'type': 'ldap'}
        method_to_call = (
            self.controller.add_security_service
            if action == 'add_security_service'
            else self.controller.remove_security_service)
        body = {
            action: {
                'security_service_id': security_service['id']
            }
        }
        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(db_api, 'security_service_get',
                         mock.Mock(return_value=security_service))
        self.mock_object(
            self.controller, '_share_network_subnets_contain_share_servers',
            mock.Mock(return_value=True))

        self.assertRaises(webob_exc.HTTPForbidden,
                          method_to_call,
                          self.req,
                          share_network['id'],
                          body)
        db_api.share_network_get.assert_called_once_with(
            self.req.environ['manila.context'], share_network['id'])
        share_networks.policy.check_policy.assert_called_once_with(
            self.req.environ['manila.context'],
            share_networks.RESOURCE_NAME, action, target_obj=share_network)

    def _setup_data_for_update_tests(self, is_check=False):
        security_services = [
            db_utils.create_security_service() for i in range(2)]
        share_network = db_utils.create_share_network()
        action = ('update_security_service_check' if is_check
                  else 'update_security_service')
        body = {
            action: {
                'current_service_id': security_services[0]['id'],
                'new_service_id': security_services[1]['id'],
            }
        }
        if is_check:
            body[action]['reset_operation'] = False
        request = fakes.HTTPRequest.blank(
            '/v2/fake/share-networks/%s/action' % share_network['id'],
            use_admin_context=True, version='2.63')
        return security_services, share_network, body, request

    def test_check_update_security_service_not_found(self):
        security_services, share_network, body, request = (
            self._setup_data_for_update_tests(is_check=True))

        context = request.environ['manila.context']

        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(db_api, 'security_service_get',
                         mock.Mock(side_effect=exception.NotFound()))

        self.assertRaises(
            webob_exc.HTTPBadRequest,
            self.controller.check_update_security_service,
            request,
            share_network['id'],
            body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id']
        )
        db_api.security_service_get.assert_called_once_with(
            context, security_services[0]['id'])

    def test_check_update_security_service(self):
        security_services, share_network, body, request = (
            self._setup_data_for_update_tests(is_check=True))
        context = request.environ['manila.context']
        share_api_return = {'fake_key': 'fake_value'}

        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(
            db_api, 'security_service_get',
            mock.Mock(
                side_effect=[security_services[0], security_services[1]]))
        self.mock_object(
            self.controller.share_api,
            'check_share_network_security_service_update',
            mock.Mock(return_vale=share_api_return))
        self.mock_object(
            self.controller._view_builder,
            'build_security_service_update_check')

        self.controller.check_update_security_service(
            request, share_network['id'], body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_has_calls(
            [mock.call(context, security_services[0]['id']),
             mock.call(context, security_services[1]['id'])])
        (self.controller.share_api.check_share_network_security_service_update.
            assert_called_once_with(
                context, share_network, security_services[1],
                current_security_service=security_services[0],
                reset_operation=False))

    @ddt.data(
        (exception.ServiceIsDown(message='fake'), webob_exc.HTTPConflict),
        (exception.InvalidShareNetwork(message='fake'),
         webob_exc.HTTPBadRequest),
        (exception.InvalidSecurityService(message='fake'),
         webob_exc.HTTPConflict))
    @ddt.unpack
    def test_check_update_security_service_share_api_failed(
            self, captured_exception, exception_to_be_raised):
        security_services, share_network, body, request = (
            self._setup_data_for_update_tests(is_check=True))
        context = request.environ['manila.context']

        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(
            db_api, 'security_service_get',
            mock.Mock(
                side_effect=[security_services[0], security_services[1]]))
        self.mock_object(
            self.controller.share_api,
            'check_share_network_security_service_update',
            mock.Mock(side_effect=captured_exception))

        self.assertRaises(
            exception_to_be_raised,
            self.controller.check_update_security_service,
            request,
            share_network['id'],
            body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_has_calls(
            [mock.call(context, security_services[0]['id']),
             mock.call(context, security_services[1]['id'])])
        (self.controller.share_api.check_share_network_security_service_update.
            assert_called_once_with(
                context, share_network, security_services[1],
                current_security_service=security_services[0],
                reset_operation=False))

    def _setup_data_for_check_add_tests(self):
        security_service = db_utils.create_security_service()
        share_network = db_utils.create_share_network()
        body = {
            'add_security_service_check': {
                'reset_operation': False,
                'security_service_id': security_service['id'],
            }
        }
        request = fakes.HTTPRequest.blank(
            '/share-networks', use_admin_context=True, version='2.63')
        return security_service, share_network, body, request

    def test_check_add_security_service_not_found(self):
        security_service, share_network, body, request = (
            self._setup_data_for_check_add_tests())

        context = request.environ['manila.context']

        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(db_api, 'security_service_get',
                         mock.Mock(side_effect=exception.NotFound()))

        self.assertRaises(
            webob_exc.HTTPBadRequest,
            self.controller.check_add_security_service,
            request,
            share_network['id'],
            body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id']
        )
        db_api.security_service_get.assert_called_once_with(
            context, security_service['id'])

    def test_check_add_security_service(self):
        security_service, share_network, body, request = (
            self._setup_data_for_check_add_tests())
        context = request.environ['manila.context']
        share_api_return = {'fake_key': 'fake_value'}

        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(
            db_api, 'security_service_get',
            mock.Mock(return_value=security_service))
        self.mock_object(
            self.controller.share_api,
            'check_share_network_security_service_update',
            mock.Mock(return_vale=share_api_return))
        self.mock_object(
            self.controller._view_builder,
            'build_security_service_update_check')

        self.controller.check_add_security_service(
            request, share_network['id'], body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_called_once_with(
            context, security_service['id'])
        (self.controller.share_api.check_share_network_security_service_update.
            assert_called_once_with(
                context, share_network, security_service,
                reset_operation=False))

    @ddt.data(
        (exception.ServiceIsDown(message='fake'), webob_exc.HTTPConflict),
        (exception.InvalidShareNetwork(message='fake'),
         webob_exc.HTTPBadRequest),
        (exception.InvalidSecurityService(message='fake'),
         webob_exc.HTTPConflict))
    @ddt.unpack
    def test_check_add_security_service_share_api_failed(
            self, captured_exception, exception_to_be_raised):
        security_service, share_network, body, request = (
            self._setup_data_for_check_add_tests())
        context = request.environ['manila.context']

        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(
            db_api, 'security_service_get',
            mock.Mock(return_value=security_service))
        self.mock_object(
            self.controller.share_api,
            'check_share_network_security_service_update',
            mock.Mock(side_effect=captured_exception))

        self.assertRaises(
            exception_to_be_raised,
            self.controller.check_add_security_service,
            request,
            share_network['id'],
            body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_called_once_with(
            context, security_service['id'])
        (self.controller.share_api.check_share_network_security_service_update.
            assert_called_once_with(
                context, share_network, security_service,
                reset_operation=False))

    @ddt.data(
        (exception.ServiceIsDown(message='fake'), webob_exc.HTTPConflict),
        (exception.InvalidShareNetwork(message='fake'),
         webob_exc.HTTPBadRequest),
        (exception.InvalidSecurityService(message='fake'),
         webob_exc.HTTPConflict))
    @ddt.unpack
    def test_update_security_service_share_api_failed(
            self, captured_exception, exception_to_be_raised):
        security_services, share_network, body, request = (
            self._setup_data_for_update_tests())
        context = request.environ['manila.context']

        self.mock_object(share_networks.policy, 'check_policy')
        self.mock_object(db_api, 'share_network_get',
                         mock.Mock(return_value=share_network))
        self.mock_object(
            db_api, 'security_service_get',
            mock.Mock(
                side_effect=[security_services[0], security_services[1]]))
        self.mock_object(
            self.controller.share_api,
            'update_share_network_security_service',
            mock.Mock(side_effect=captured_exception))

        self.assertRaises(exception_to_be_raised,
                          self.controller.update_security_service,
                          request,
                          share_network['id'],
                          body)

        db_api.share_network_get.assert_called_once_with(
            context, share_network['id'])
        db_api.security_service_get.assert_has_calls(
            [mock.call(context, security_services[0]['id']),
             mock.call(context, security_services[1]['id'])])
        (self.controller.share_api.update_share_network_security_service.
            assert_called_once_with(
                context, share_network, security_services[1],
                current_security_service=security_services[0]))

    def test_reset_status(self):
        share_network = db_utils.create_share_network()
        request = fakes.HTTPRequest.blank(
            '/v2/fake/share-networks/%s/action' % share_network['id'],
            use_admin_context=True, version='2.63')
        self.mock_object(db_api, 'share_network_update')

        status = {'status': 'active'}
        body = {'reset_status': status}

        response = self.controller.reset_status(request,
                                                share_network['id'], body)

        self.assertEqual(202, response.status_int)
        db_api.share_network_update.assert_called_once_with(
            request.environ['manila.context'],
            share_network['id'], {'status': 'active'})



# Copyright (c) 2015 Hitachi Data Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt
from oslo_config import cfg

from manila import exception
import manila.share.configuration
import manila.share.driver
from manila.share.drivers.hitachi.hnas import driver
from manila.share.drivers.hitachi.hnas import ssh
from manila import test

CONF = cfg.CONF

share_nfs = {
    'id': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'name': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'size': 50,
    'host': 'hnas',
    'share_proto': 'NFS',
    'share_type_id': 1,
    'share_network_id': 'bb329e24-3bdb-491d-acfd-dfe70c09b98d',
    'share_server_id': 'cc345a53-491d-acfd-3bdb-dfe70c09b98d',
    'export_locations': [{'path': '172.24.44.10:/shares/'
                                  'aa4a7710-f326-41fb-ad18-b4ad587fc87a'}],
}

share_cifs = {
    'id': 'f5cadaf2-afbe-4cc4-9021-85491b6b76f7',
    'name': 'f5cadaf2-afbe-4cc4-9021-85491b6b76f7',
    'size': 50,
    'host': 'hnas',
    'share_proto': 'CIFS',
    'share_type_id': 1,
    'share_network_id': 'bb329e24-3bdb-491d-acfd-dfe70c09b98d',
    'share_server_id': 'cc345a53-491d-acfd-3bdb-dfe70c09b98d',
    'export_locations': [{'path': '\\\\172.24.44.10\\'
                                  'f5cadaf2-afbe-4cc4-9021-85491b6b76f7'}],
}

share_invalid_host = {
    'id': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'name': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'size': 50,
    'host': 'invalid',
    'share_proto': 'NFS',
    'share_type_id': 1,
    'share_network_id': 'bb329e24-3bdb-491d-acfd-dfe70c09b98d',
    'share_server_id': 'cc345a53-491d-acfd-3bdb-dfe70c09b98d',
    'export_locations': [{'path': '172.24.44.10:/shares/'
                                  'aa4a7710-f326-41fb-ad18-b4ad587fc87a'}],
}

share_mount_support_nfs = {
    'id': '62125744-fcdd-4f55-a8c1-d1498102f634',
    'name': '62125744-fcdd-4f55-a8c1-d1498102f634',
    'size': 50,
    'host': 'hnas',
    'share_proto': 'NFS',
    'share_type_id': 1,
    'share_network_id': 'bb329e24-3bdb-491d-acfd-dfe70c09b98d',
    'share_server_id': 'cc345a53-491d-acfd-3bdb-dfe70c09b98d',
    'export_locations': [{'path': '172.24.44.10:/shares/'
                                  '62125744-fcdd-4f55-a8c1-d1498102f634'}],
    'mount_snapshot_support': True,
}

share_mount_support_cifs = {
    'id': 'd6e7dc6b-f65f-49d9-968d-936f75474f29',
    'name': 'd6e7dc6b-f65f-49d9-968d-936f75474f29',
    'size': 50,
    'host': 'hnas',
    'share_proto': 'CIFS',
    'share_type_id': 1,
    'share_network_id': 'bb329e24-3bdb-491d-acfd-dfe70c09b98d',
    'share_server_id': 'cc345a53-491d-acfd-3bdb-dfe70c09b98d',
    'export_locations': [{'path': '172.24.44.10:/shares/'
                                  'd6e7dc6b-f65f-49d9-968d-936f75474f29'}],
    'mount_snapshot_support': True,
}

access_nfs_rw = {
    'id': 'acdc7172b-fe07-46c4-b78f-df3e0324ccd0',
    'access_type': 'ip',
    'access_to': '172.24.44.200',
    'access_level': 'rw',
    'state': 'active',
}

access_cifs_rw = {
    'id': '43167594-40e9-b899-1f4f-b9c2176b7564',
    'access_type': 'user',
    'access_to': 'fake_user',
    'access_level': 'rw',
    'state': 'active',
}

access_cifs_ro = {
    'id': '32407088-1f4f-40e9-b899-b9a4176b574d',
    'access_type': 'user',
    'access_to': 'fake_user',
    'access_level': 'ro',
    'state': 'active',
}

snapshot_nfs = {
    'id': 'abba6d9b-f29c-4bf7-aac1-618cda7aaf0f',
    'share_id': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'share': share_nfs,
    'provider_location': '/snapshots/aa4a7710-f326-41fb-ad18-b4ad587fc87a/'
                         'abba6d9b-f29c-4bf7-aac1-618cda7aaf0f',
    'size': 2,
}

snapshot_cifs = {
    'id': '91bc6e1b-1ba5-f29c-abc1-da7618cabf0a',
    'share_id': 'f5cadaf2-afbe-4cc4-9021-85491b6b76f7',
    'share': share_cifs,
    'provider_location': '/snapshots/f5cadaf2-afbe-4cc4-9021-85491b6b76f7/'
                         '91bc6e1b-1ba5-f29c-abc1-da7618cabf0a',
    'size': 2,
}

manage_snapshot = {
    'id': 'bc168eb-fa71-beef-153a-3d451aa1351f',
    'share_id': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'share': share_nfs,
    'provider_location': '/snapshots/aa4a7710-f326-41fb-ad18-b4ad587fc87a'
                         '/snapshot18-05-2106',
}

snapshot_mount_support_nfs = {
    'id': '3377b015-a695-4a5a-8aa5-9b931b023380',
    'share_id': '62125744-fcdd-4f55-a8c1-d1498102f634',
    'share': share_mount_support_nfs,
    'provider_location': '/snapshots/62125744-fcdd-4f55-a8c1-d1498102f634'
                         '/3377b015-a695-4a5a-8aa5-9b931b023380',
}

snapshot_mount_support_cifs = {
    'id': 'f9916515-5cb8-4612-afa6-7f2baa74223a',
    'share_id': 'd6e7dc6b-f65f-49d9-968d-936f75474f29',
    'share': share_mount_support_cifs,
    'provider_location': '/snapshots/d6e7dc6b-f65f-49d9-968d-936f75474f29'
                         '/f9916515-5cb8-4612-afa6-7f2baa74223a',
}

invalid_share = {
    'id': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'name': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'size': 100,
    'host': 'hnas',
    'share_proto': 'HDFS',
}

invalid_snapshot = {
    'id': '24dcdcb5-a582-4bcc-b462-641da143afee',
    'share_id': 'aa4a7710-f326-41fb-ad18-b4ad587fc87a',
    'share': invalid_share,
}

invalid_access_type = {
    'id': 'acdc7172b-fe07-46c4-b78f-df3e0324ccd0',
    'access_type': 'cert',
    'access_to': 'manila_user',
    'access_level': 'rw',
    'state': 'active',
}

invalid_access_level = {
    'id': 'acdc7172b-fe07-46c4-b78f-df3e0324ccd0',
    'access_type': 'ip',
    'access_to': 'manila_user',
    'access_level': '777',
    'state': 'active',
}

invalid_protocol_msg = ("Share backend error: Only NFS or CIFS protocol are "
                        "currently supported. Share provided %(id)s with "
                        "protocol %(proto)s." %
                        {'id': invalid_share['id'],
                         'proto': invalid_share['share_proto']})


@ddt.ddt
class HitachiHNASTestCase(test.TestCase):
    def setUp(self):
        super(HitachiHNASTestCase, self).setUp()
        CONF.set_default('driver_handles_share_servers', False)
        CONF.hitachi_hnas_evs_id = '2'
        CONF.hitachi_hnas_evs_ip = '172.24.44.10'
        CONF.hitachi_hnas_admin_network_ip = '10.20.30.40'
        CONF.hitachi_hnas_ip = '172.24.44.1'
        CONF.hitachi_hnas_ip_port = 'hitachi_hnas_ip_port'
        CONF.hitachi_hnas_user = 'hitachi_hnas_user'
        CONF.hitachi_hnas_password = 'hitachi_hnas_password'
        CONF.hitachi_hnas_file_system_name = 'file_system'
        CONF.hitachi_hnas_ssh_private_key = 'private_key'
        CONF.hitachi_hnas_cluster_admin_ip0 = None
        CONF.hitachi_hnas_stalled_job_timeout = 10
        CONF.hitachi_hnas_driver_helper = ('manila.share.drivers.hitachi.hnas.'
                                           'ssh.HNASSSHBackend')
        self.fake_conf = manila.share.configuration.Configuration(None)

        self.fake_private_storage = mock.Mock()
        self.mock_object(self.fake_private_storage, 'get',
                         mock.Mock(return_value=None))
        self.mock_object(self.fake_private_storage, 'delete',
                         mock.Mock(return_value=None))

        self._driver = driver.HitachiHNASDriver(
            private_storage=self.fake_private_storage,
            configuration=self.fake_conf)
        self._driver.backend_name = "hnas"
        self.mock_log = self.mock_object(driver, 'LOG')

        # mocking common backend calls
        self.mock_object(ssh.HNASSSHBackend, "check_fs_mounted", mock.Mock(
            return_value=True))
        self.mock_object(ssh.HNASSSHBackend, "check_vvol")
        self.mock_object(ssh.HNASSSHBackend, "check_quota")
        self.mock_object(ssh.HNASSSHBackend, "check_cifs")
        self.mock_object(ssh.HNASSSHBackend, "check_export")
        self.mock_object(ssh.HNASSSHBackend, 'check_directory')

    @ddt.data('hitachi_hnas_driver_helper', 'hitachi_hnas_evs_id',
              'hitachi_hnas_evs_ip', 'hitachi_hnas_ip', 'hitachi_hnas_user')
    def test_init_invalid_conf_parameters(self, attr_name):
        self.mock_object(manila.share.driver.ShareDriver, '__init__')
        setattr(CONF, attr_name, None)

        self.assertRaises(exception.InvalidParameterValue,
                          self._driver.__init__)

    def test_init_invalid_credentials(self):
        self.mock_object(manila.share.driver.ShareDriver,
                         '__init__')
        CONF.hitachi_hnas_password = None
        CONF.hitachi_hnas_ssh_private_key = None

        self.assertRaises(exception.InvalidParameterValue,
                          self._driver.__init__)

    @ddt.data(True, False)
    def test_update_access_nfs(self, empty_rules):
        if not empty_rules:
            access1 = {
                'access_type': 'ip',
                'access_to': '172.24.10.10',
                'access_level': 'rw'
            }
            access2 = {
                'access_type': 'ip',
                'access_to': '188.100.20.10',
                'access_level': 'ro'
            }
            access_list = [access1, access2]
            access_list_updated = (
                [access1['access_to'] + '(' + access1['access_level'] +
                 ',norootsquash)', access2['access_to'] + '(' +
                 access2['access_level'] + ')', ])
        else:
            access_list = []
            access_list_updated = []

        self.mock_object(ssh.HNASSSHBackend, "update_nfs_access_rule",
                         mock.Mock())
        self._driver.update_access('context', share_nfs, access_list, [], [])

        ssh.HNASSSHBackend.update_nfs_access_rule.assert_called_once_with(
            access_list_updated, share_id=share_nfs['id'])
        self.assertTrue(self.mock_log.debug.called)

    def test_update_access_ip_exception(self):
        access1 = {
            'access_type': 'ip',
            'access_to': '188.100.20.10',
            'access_level': 'ro'
        }
        access2 = {
            'access_type': 'something',
            'access_to': '172.24.10.10',
            'access_level': 'rw'
        }
        access_list = [access1, access2]

        self.assertRaises(exception.InvalidShareAccess,
                          self._driver.update_access, 'context', share_nfs,
                          access_list, [], [])

    def test_update_access_not_found_exception(self):
        access1 = {
            'access_type': 'ip',
            'access_to': '188.100.20.10',
            'access_level': 'ro'
        }
        access2 = {
            'access_type': 'something',
            'access_to': '172.24.10.10',
            'access_level': 'rw'
        }
        access_list = [access1, access2]

        self.mock_object(self._driver, '_ensure_share', mock.Mock(
            side_effect=exception.HNASItemNotFoundException(msg='fake')))

        self.assertRaises(exception.ShareResourceNotFound,
                          self._driver.update_access, 'context', share_nfs,
                          access_list, add_rules=[], delete_rules=[])

    @ddt.data([access_cifs_rw, 'acr'], [access_cifs_ro, 'ar'])
    @ddt.unpack
    def test_allow_access_cifs(self, access_cifs, permission):
        access_list_allow = [access_cifs]

        self.mock_object(ssh.HNASSSHBackend, 'cifs_allow_access')

        self._driver.update_access('context', share_cifs, [],
                                   access_list_allow, [])

        ssh.HNASSSHBackend.cifs_allow_access.assert_called_once_with(
            share_cifs['id'], 'fake_user', permission, is_snapshot=False)
        self.assertTrue(self.mock_log.debug.called)

    def test_allow_access_cifs_invalid_type(self):
        access_cifs_type_ip = {
            'id': '43167594-40e9-b899-1f4f-b9c2176b7564',
            'access_type': 'ip',
            'access_to': 'fake_user',
            'access_level': 'rw',
            'state': 'active',
        }
        access_list_allow = [access_cifs_type_ip]

        self.assertRaises(exception.InvalidShareAccess,
                          self._driver.update_access, 'context', share_cifs,
                          [], access_list_allow, [])

    def test_deny_access_cifs(self):
        access_list_deny = [access_cifs_rw]

        self.mock_object(ssh.HNASSSHBackend, 'cifs_deny_access')

        self._driver.update_access('context', share_cifs, [], [],
                                   access_list_deny)

        ssh.HNASSSHBackend.cifs_deny_access.assert_called_once_with(
            share_cifs['id'], 'fake_user', is_snapshot=False)
        self.assertTrue(self.mock_log.debug.called)

    def test_deny_access_cifs_unsupported_type(self):
        access_cifs_type_ip = {
            'id': '43167594-40e9-b899-1f4f-b9c2176b7564',
            'access_type': 'ip',
            'access_to': 'fake_user',
            'access_level': 'rw',
            'state': 'active',
        }
        access_list_deny = [access_cifs_type_ip]

        self.mock_object(ssh.HNASSSHBackend, 'cifs_deny_access')

        self._driver.update_access('context', share_cifs, [], [],
                                   access_list_deny)
        self.assertTrue(self.mock_log.warning.called)

    def test_update_access_invalid_share_protocol(self):
        self.mock_object(self._driver, '_ensure_share')
        ex = self.assertRaises(exception.ShareBackendException,
                               self._driver.update_access, 'context',
                               invalid_share, [], [], [])
        self.assertEqual(invalid_protocol_msg, ex.msg)

    def test_update_access_cifs_recovery_mode(self):
        access_list = [access_cifs_rw, access_cifs_ro]
        permission_list = [('fake_user1', 'acr'), ('fake_user2', 'ar')]

        self.mock_object(ssh.HNASSSHBackend, 'list_cifs_permissions',
                         mock.Mock(return_value=permission_list))
        self.mock_object(ssh.HNASSSHBackend, 'cifs_deny_access')
        self.mock_object(ssh.HNASSSHBackend, 'cifs_allow_access')

        self._driver.update_access('context', share_cifs, access_list, [], [])

        ssh.HNASSSHBackend.list_cifs_permissions.assert_called_once_with(
            share_cifs['id'])
        self.assertTrue(self.mock_log.debug.called)

    def _get_export(self, id, share_proto, ip, is_admin_only,
                    is_snapshot=False):
        if share_proto.lower() == 'nfs':
            if is_snapshot:
                path = '/snapshots/' + id
            else:
                path = '/shares/' + id
            export = ':'.join((ip, path))
        else:
            export = r'\\%s\%s' % (ip, id)

        return {
            "path": export,
            "is_admin_only": is_admin_only,
            "metadata": {},
        }

    @ddt.data(share_nfs, share_cifs)
    def test_create_share(self, share):
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "vvol_create")
        self.mock_object(ssh.HNASSSHBackend, "quota_add")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_add", mock.Mock(
            return_value='/shares/' + share['id']))
        self.mock_object(ssh.HNASSSHBackend, "cifs_share_add")

        result = self._driver.create_share('context', share)

        self.assertTrue(self.mock_log.debug.called)
        ssh.HNASSSHBackend.vvol_create.assert_called_once_with(share['id'])
        ssh.HNASSSHBackend.quota_add.assert_called_once_with(share['id'],
                                                             share['size'])
        expected = [
            self._get_export(
                share['id'], share['share_proto'], self._driver.hnas_evs_ip,
                False),
            self._get_export(
                share['id'], share['share_proto'],
                self._driver.hnas_admin_network_ip, True)]

        if share['share_proto'].lower() == 'nfs':
            ssh.HNASSSHBackend.nfs_export_add.assert_called_once_with(
                share_nfs['id'], snapshot_id=None)
            self.assertFalse(ssh.HNASSSHBackend.cifs_share_add.called)
        else:
            ssh.HNASSSHBackend.cifs_share_add.assert_called_once_with(
                share_cifs['id'], snapshot_id=None)
            self.assertFalse(ssh.HNASSSHBackend.nfs_export_add.called)
        self.assertEqual(expected, result)

    def test_create_share_export_error(self):
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "vvol_create")
        self.mock_object(ssh.HNASSSHBackend, "quota_add")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_add", mock.Mock(
            side_effect=exception.HNASBackendException('msg')))
        self.mock_object(ssh.HNASSSHBackend, "vvol_delete")

        self.assertRaises(exception.HNASBackendException,
                          self._driver.create_share, 'context', share_nfs)
        self.assertTrue(self.mock_log.debug.called)
        ssh.HNASSSHBackend.vvol_create.assert_called_once_with(share_nfs['id'])
        ssh.HNASSSHBackend.quota_add.assert_called_once_with(share_nfs['id'],
                                                             share_nfs['size'])
        ssh.HNASSSHBackend.nfs_export_add.assert_called_once_with(
            share_nfs['id'], snapshot_id=None)
        ssh.HNASSSHBackend.vvol_delete.assert_called_once_with(share_nfs['id'])

    def test_create_share_invalid_share_protocol(self):
        self.mock_object(driver.HitachiHNASDriver, "_create_share",
                         mock.Mock(return_value="path"))

        ex = self.assertRaises(exception.ShareBackendException,
                               self._driver.create_share, 'context',
                               invalid_share)
        self.assertEqual(invalid_protocol_msg, ex.msg)

    @ddt.data(share_nfs, share_cifs)
    def test_delete_share(self, share):
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_del")
        self.mock_object(ssh.HNASSSHBackend, "cifs_share_del")
        self.mock_object(ssh.HNASSSHBackend, "vvol_delete")

        self._driver.delete_share('context', share)

        self.assertTrue(self.mock_log.debug.called)
        ssh.HNASSSHBackend.vvol_delete.assert_called_once_with(share['id'])

        if share['share_proto'].lower() == 'nfs':
            ssh.HNASSSHBackend.nfs_export_del.assert_called_once_with(
                share['id'])
            self.assertFalse(ssh.HNASSSHBackend.cifs_share_del.called)
        else:
            ssh.HNASSSHBackend.cifs_share_del.assert_called_once_with(
                share['id'])
            self.assertFalse(ssh.HNASSSHBackend.nfs_export_del.called)

    @ddt.data(snapshot_nfs, snapshot_cifs, snapshot_mount_support_nfs,
              snapshot_mount_support_cifs)
    def test_create_snapshot(self, snapshot):
        hnas_id = snapshot['share_id']
        access_list = ['172.24.44.200(rw,norootsquash)',
                       '172.24.49.180(all_squash,read_write,secure)',
                       '172.24.49.110(ro, secure)',
                       '172.24.49.112(secure,readwrite,norootsquash)',
                       '172.24.49.142(read_only, secure)',
                       '172.24.49.201(rw,read_write,readwrite)',
                       '172.24.49.218(rw)']

        ro_list = ['172.24.44.200(ro,norootsquash)',
                   '172.24.49.180(all_squash,ro,secure)',
                   '172.24.49.110(ro, secure)',
                   '172.24.49.112(secure,ro,norootsquash)',
                   '172.24.49.142(read_only, secure)',
                   '172.24.49.201(ro,ro,ro)',
                   '172.24.49.218(ro)']

        export_locations = [
            self._get_export(
                snapshot['id'], snapshot['share']['share_proto'],
                self._driver.hnas_evs_ip, False, is_snapshot=True),
            self._get_export(
                snapshot['id'], snapshot['share']['share_proto'],
                self._driver.hnas_admin_network_ip, True, is_snapshot=True)]

        expected = {'provider_location': '/snapshots/' + hnas_id + '/' +
                                         snapshot['id']}

        if snapshot['share'].get('mount_snapshot_support'):
            expected['export_locations'] = export_locations

        self.mock_object(ssh.HNASSSHBackend, "get_nfs_host_list", mock.Mock(
            return_value=access_list))
        self.mock_object(ssh.HNASSSHBackend, "update_nfs_access_rule",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "is_cifs_in_use", mock.Mock(
            return_value=False))
        self.mock_object(ssh.HNASSSHBackend, "tree_clone")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_add")
        self.mock_object(ssh.HNASSSHBackend, "cifs_share_add")

        out = self._driver.create_snapshot('context', snapshot)

        ssh.HNASSSHBackend.tree_clone.assert_called_once_with(
            '/shares/' + hnas_id, '/snapshots/' + hnas_id + '/' +
            snapshot['id'])
        self.assertEqual(expected, out)

        if snapshot['share']['share_proto'].lower() == 'nfs':
            ssh.HNASSSHBackend.get_nfs_host_list.assert_called_once_with(
                hnas_id)
            ssh.HNASSSHBackend.update_nfs_access_rule.assert_any_call(
                ro_list, share_id=hnas_id)
            ssh.HNASSSHBackend.update_nfs_access_rule.assert_any_call(
                access_list, share_id=hnas_id)
        else:
            ssh.HNASSSHBackend.is_cifs_in_use.assert_called_once_with(
                hnas_id)

    def test_create_snapshot_invalid_protocol(self):
        self.mock_object(self._driver, '_ensure_share')
        ex = self.assertRaises(exception.ShareBackendException,
                               self._driver.create_snapshot, 'context',
                               invalid_snapshot)
        self.assertEqual(invalid_protocol_msg, ex.msg)

    def test_create_snapshot_cifs_exception(self):
        cifs_excep_msg = ("Share backend error: CIFS snapshot when share is "
                          "mounted is disabled. Set "
                          "hitachi_hnas_allow_cifs_snapshot_while_mounted to "
                          "True or unmount the share to take a snapshot.")

        self.mock_object(ssh.HNASSSHBackend, "is_cifs_in_use", mock.Mock(
            return_value=True))

        ex = self.assertRaises(exception.ShareBackendException,
                               self._driver.create_snapshot, 'context',
                               snapshot_cifs)
        self.assertEqual(cifs_excep_msg, ex.msg)

    def test_create_snapshot_first_snapshot(self):
        hnas_id = snapshot_nfs['share_id']
        self.mock_object(ssh.HNASSSHBackend, "get_nfs_host_list", mock.Mock(
            return_value=['172.24.44.200(rw)']))
        self.mock_object(ssh.HNASSSHBackend, "update_nfs_access_rule",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "tree_clone", mock.Mock(
            side_effect=exception.HNASNothingToCloneException('msg')))
        self.mock_object(ssh.HNASSSHBackend, "create_directory")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_add")
        self.mock_object(ssh.HNASSSHBackend, "cifs_share_add")

        self._driver.create_snapshot('context', snapshot_nfs)

        self.assertTrue(self.mock_log.warning.called)
        ssh.HNASSSHBackend.get_nfs_host_list.assert_called_once_with(
            hnas_id)
        ssh.HNASSSHBackend.update_nfs_access_rule.assert_any_call(
            ['172.24.44.200(ro)'], share_id=hnas_id)
        ssh.HNASSSHBackend.update_nfs_access_rule.assert_any_call(
            ['172.24.44.200(rw)'], share_id=hnas_id)
        ssh.HNASSSHBackend.create_directory.assert_called_once_with(
            '/snapshots/' + hnas_id + '/' + snapshot_nfs['id'])

    @ddt.data(snapshot_nfs, snapshot_cifs,
              snapshot_mount_support_nfs, snapshot_mount_support_cifs)
    def test_delete_snapshot(self, snapshot):
        hnas_share_id = snapshot['share_id']
        hnas_snapshot_id = snapshot['id']
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted")
        self.mock_object(ssh.HNASSSHBackend, "tree_delete")
        self.mock_object(ssh.HNASSSHBackend, "delete_directory")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_del")
        self.mock_object(ssh.HNASSSHBackend, "cifs_share_del")

        self._driver.delete_snapshot('context', snapshot)

        self.assertTrue(self.mock_log.debug.called)
        self.assertTrue(self.mock_log.info.called)
        driver.HitachiHNASDriver._check_fs_mounted.assert_called_once_with()
        ssh.HNASSSHBackend.tree_delete.assert_called_once_with(
            '/snapshots/' + hnas_share_id + '/' + snapshot['id'])
        ssh.HNASSSHBackend.delete_directory.assert_called_once_with(
            '/snapshots/' + hnas_share_id)
        if snapshot['share']['share_proto'].lower() == 'nfs':
            if snapshot['share'].get('mount_snapshot_support'):
                ssh.HNASSSHBackend.nfs_export_del.assert_called_once_with(
                    snapshot_id=hnas_snapshot_id)
            else:
                ssh.HNASSSHBackend.nfs_export_del.assert_not_called()
        else:
            if snapshot['share'].get('mount_snapshot_support'):
                ssh.HNASSSHBackend.cifs_share_del.assert_called_once_with(
                    hnas_snapshot_id)
            else:
                ssh.HNASSSHBackend.cifs_share_del.assert_not_called()

    def test_delete_managed_snapshot(self):
        hnas_id = manage_snapshot['share_id']
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted")
        self.mock_object(ssh.HNASSSHBackend, "tree_delete")
        self.mock_object(ssh.HNASSSHBackend, "delete_directory")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_del")
        self.mock_object(ssh.HNASSSHBackend, "cifs_share_del")

        self._driver.delete_snapshot('context', manage_snapshot)

        self.assertTrue(self.mock_log.debug.called)
        self.assertTrue(self.mock_log.info.called)
        driver.HitachiHNASDriver._check_fs_mounted.assert_called_once_with()
        ssh.HNASSSHBackend.tree_delete.assert_called_once_with(
            manage_snapshot['provider_location'])
        ssh.HNASSSHBackend.delete_directory.assert_called_once_with(
            '/snapshots/' + hnas_id)

    @ddt.data(share_nfs, share_cifs)
    def test_ensure_share(self, share):
        result = self._driver.ensure_share('context', share)

        ssh.HNASSSHBackend.check_vvol.assert_called_once_with(share['id'])
        ssh.HNASSSHBackend.check_quota.assert_called_once_with(share['id'])

        expected = [
            self._get_export(
                share['id'], share['share_proto'], self._driver.hnas_evs_ip,
                False),
            self._get_export(
                share['id'], share['share_proto'],
                self._driver.hnas_admin_network_ip, True)]

        if share['share_proto'].lower() == 'nfs':
            ssh.HNASSSHBackend.check_export.assert_called_once_with(
                share['id'])
            self.assertFalse(ssh.HNASSSHBackend.check_cifs.called)
        else:
            ssh.HNASSSHBackend.check_cifs.assert_called_once_with(share['id'])
            self.assertFalse(ssh.HNASSSHBackend.check_export.called)
        self.assertEqual(expected, result)

    def test_ensure_share_invalid_protocol(self):
        ex = self.assertRaises(exception.ShareBackendException,
                               self._driver.ensure_share, 'context',
                               invalid_share)

        self.assertEqual(invalid_protocol_msg, ex.msg)

    def test_shrink_share(self):
        self.mock_object(ssh.HNASSSHBackend, "get_share_usage", mock.Mock(
            return_value=10))
        self.mock_object(ssh.HNASSSHBackend, "modify_quota")

        self._driver.shrink_share(share_nfs, 11)

        ssh.HNASSSHBackend.get_share_usage.assert_called_once_with(
            share_nfs['id'])
        ssh.HNASSSHBackend.modify_quota.assert_called_once_with(
            share_nfs['id'], 11)

    def test_shrink_share_new_size_lower_than_usage(self):
        self.mock_object(ssh.HNASSSHBackend, "get_share_usage", mock.Mock(
            return_value=10))

        self.assertRaises(exception.ShareShrinkingPossibleDataLoss,
                          self._driver.shrink_share, share_nfs, 9)
        ssh.HNASSSHBackend.get_share_usage.assert_called_once_with(
            share_nfs['id'])

    def test_extend_share(self):
        self.mock_object(ssh.HNASSSHBackend, "get_stats", mock.Mock(
            return_value=(500, 200, True)))
        self.mock_object(ssh.HNASSSHBackend, "modify_quota")

        self._driver.extend_share(share_nfs, 150)

        ssh.HNASSSHBackend.get_stats.assert_called_once_with()
        ssh.HNASSSHBackend.modify_quota.assert_called_once_with(
            share_nfs['id'], 150)

    def test_extend_share_with_no_available_space_in_fs(self):
        self.mock_object(ssh.HNASSSHBackend, "get_stats", mock.Mock(
            return_value=(500, 200, False)))
        self.mock_object(ssh.HNASSSHBackend, "modify_quota")

        self.assertRaises(exception.HNASBackendException,
                          self._driver.extend_share, share_nfs, 1000)
        ssh.HNASSSHBackend.get_stats.assert_called_once_with()

    @ddt.data(share_nfs, share_cifs)
    def test_manage_existing(self, share):

        expected_exports = [
            self._get_export(
                share['id'], share['share_proto'], self._driver.hnas_evs_ip,
                False),
            self._get_export(
                share['id'], share['share_proto'],
                self._driver.hnas_admin_network_ip, True)]

        expected_out = {'size': share['size'],
                        'export_locations': expected_exports}

        self.mock_object(ssh.HNASSSHBackend, "get_share_quota", mock.Mock(
            return_value=share['size']))

        out = self._driver.manage_existing(share, 'option')

        self.assertEqual(expected_out, out)
        ssh.HNASSSHBackend.get_share_quota.assert_called_once_with(
            share['id'])

    def test_manage_existing_no_quota(self):
        self.mock_object(ssh.HNASSSHBackend, "get_share_quota", mock.Mock(
            return_value=None))

        self.assertRaises(exception.ManageInvalidShare,
                          self._driver.manage_existing, share_nfs, 'option')
        ssh.HNASSSHBackend.get_share_quota.assert_called_once_with(
            share_nfs['id'])

    def test_manage_existing_wrong_share_id(self):
        self.mock_object(self.fake_private_storage, 'get',
                         mock.Mock(return_value='Wrong_share_id'))

        self.assertRaises(exception.HNASBackendException,
                          self._driver.manage_existing, share_nfs, 'option')

    @ddt.data(':/', '1.1.1.1:/share_id', '1.1.1.1:/shares',
              '1.1.1.1:shares/share_id', ':/share_id')
    def test_manage_existing_wrong_path_format_nfs(self, wrong_location):
        expected_exception = ("Share backend error: Incorrect path. It "
                              "should have the following format: "
                              "IP:/shares/share_id.")

        self._test_manage_existing_wrong_path(
            share_nfs.copy(), expected_exception, wrong_location)

    @ddt.data('\\\\1.1.1.1', '1.1.1.1\\share_id', '1.1.1.1\\shares\\share_id',
              '\\\\1.1.1.1\\shares\\share_id', '\\\\share_id')
    def test_manage_existing_wrong_path_format_cifs(self, wrong_location):
        expected_exception = ("Share backend error: Incorrect path. It should "
                              "have the following format: \\\\IP\\share_id.")

        self._test_manage_existing_wrong_path(
            share_cifs.copy(), expected_exception, wrong_location)

    def _test_manage_existing_wrong_path(
            self, share, expected_exception, wrong_location):

        share['export_locations'] = [{'path': wrong_location}]

        ex = self.assertRaises(exception.ShareBackendException,
                               self._driver.manage_existing, share, 'option')

        self.assertEqual(expected_exception, ex.msg)

    def test_manage_existing_wrong_evs_ip(self):
        share_nfs['export_locations'] = [{'path': '172.24.44.189:/shares/'
                                                  'aa4a7710-f326-41fb-ad18-'}]

        self.assertRaises(exception.ShareBackendException,
                          self._driver.manage_existing, share_nfs,
                          'option')

    def test_manage_existing_invalid_host(self):
        self.assertRaises(exception.ShareBackendException,
                          self._driver.manage_existing, share_invalid_host,
                          'option')

    def test_manage_existing_invalid_protocol(self):
        self.assertRaises(exception.ShareBackendException,
                          self._driver.manage_existing, invalid_share,
                          'option')

    @ddt.data(True, False)
    def test_unmanage(self, has_export_locations):
        share_copy = share_nfs.copy()
        if not has_export_locations:
            share_copy['export_locations'] = []

        self._driver.unmanage(share_copy)

        self.assertTrue(self.fake_private_storage.delete.called)
        self.assertTrue(self.mock_log.info.called)

    def test_get_network_allocations_number(self):
        result = self._driver.get_network_allocations_number()

        self.assertEqual(0, result)

    @ddt.data([share_nfs, snapshot_nfs], [share_cifs, snapshot_cifs])
    @ddt.unpack
    def test_create_share_from_snapshot(self, share, snapshot):
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "vvol_create")
        self.mock_object(ssh.HNASSSHBackend, "quota_add")
        self.mock_object(ssh.HNASSSHBackend, "tree_clone")
        self.mock_object(ssh.HNASSSHBackend, "cifs_share_add")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_add")

        result = self._driver.create_share_from_snapshot('context',
                                                         share,
                                                         snapshot)

        ssh.HNASSSHBackend.vvol_create.assert_called_once_with(share['id'])
        ssh.HNASSSHBackend.quota_add.assert_called_once_with(share['id'],
                                                             share['size'])
        ssh.HNASSSHBackend.tree_clone.assert_called_once_with(
            '/snapshots/' + share['id'] + '/' + snapshot['id'],
            '/shares/' + share['id'])

        expected = [
            self._get_export(
                share['id'], share['share_proto'], self._driver.hnas_evs_ip,
                False),
            self._get_export(
                share['id'], share['share_proto'],
                self._driver.hnas_admin_network_ip, True)]

        if share['share_proto'].lower() == 'nfs':
            ssh.HNASSSHBackend.nfs_export_add.assert_called_once_with(
                share['id'])
            self.assertFalse(ssh.HNASSSHBackend.cifs_share_add.called)
        else:
            ssh.HNASSSHBackend.cifs_share_add.assert_called_once_with(
                share['id'])
            self.assertFalse(ssh.HNASSSHBackend.nfs_export_add.called)

        self.assertEqual(expected, result)

    def test_create_share_from_snapshot_empty_snapshot(self):
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "vvol_create")
        self.mock_object(ssh.HNASSSHBackend, "quota_add")
        self.mock_object(ssh.HNASSSHBackend, "tree_clone", mock.Mock(
            side_effect=exception.HNASNothingToCloneException('msg')))
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_add")

        result = self._driver.create_share_from_snapshot('context', share_nfs,
                                                         snapshot_nfs)
        expected = [
            self._get_export(
                share_nfs['id'], share_nfs['share_proto'],
                self._driver.hnas_evs_ip, False),
            self._get_export(
                share_nfs['id'], share_nfs['share_proto'],
                self._driver.hnas_admin_network_ip, True)]

        self.assertEqual(expected, result)
        self.assertTrue(self.mock_log.warning.called)
        ssh.HNASSSHBackend.vvol_create.assert_called_once_with(share_nfs['id'])
        ssh.HNASSSHBackend.quota_add.assert_called_once_with(share_nfs['id'],
                                                             share_nfs['size'])
        ssh.HNASSSHBackend.tree_clone.assert_called_once_with(
            '/snapshots/' + share_nfs['id'] + '/' + snapshot_nfs['id'],
            '/shares/' + share_nfs['id'])
        ssh.HNASSSHBackend.nfs_export_add.assert_called_once_with(
            share_nfs['id'])

    def test_create_share_from_snapshot_invalid_protocol(self):
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "vvol_create")
        self.mock_object(ssh.HNASSSHBackend, "quota_add")
        self.mock_object(ssh.HNASSSHBackend, "tree_clone")

        ex = self.assertRaises(exception.ShareBackendException,
                               self._driver.create_share_from_snapshot,
                               'context', invalid_share, snapshot_nfs)
        self.assertEqual(invalid_protocol_msg, ex.msg)

    def test_create_share_from_snapshot_cleanup(self):
        dest_path = '/snapshots/' + share_nfs['id'] + '/' + snapshot_nfs['id']
        src_path = '/shares/' + share_nfs['id']

        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(ssh.HNASSSHBackend, "vvol_create")
        self.mock_object(ssh.HNASSSHBackend, "quota_add")
        self.mock_object(ssh.HNASSSHBackend, "tree_clone")
        self.mock_object(ssh.HNASSSHBackend, "vvol_delete")
        self.mock_object(ssh.HNASSSHBackend, "nfs_export_add", mock.Mock(
            side_effect=exception.HNASBackendException(
                msg='Error adding nfs export.')))

        self.assertRaises(exception.HNASBackendException,
                          self._driver.create_share_from_snapshot,
                          'context', share_nfs, snapshot_nfs)

        ssh.HNASSSHBackend.vvol_create.assert_called_once_with(
            share_nfs['id'])
        ssh.HNASSSHBackend.quota_add.assert_called_once_with(
            share_nfs['id'], share_nfs['size'])
        ssh.HNASSSHBackend.tree_clone.assert_called_once_with(
            dest_path, src_path)
        ssh.HNASSSHBackend.nfs_export_add.assert_called_once_with(
            share_nfs['id'])
        ssh.HNASSSHBackend.vvol_delete.assert_called_once_with(
            share_nfs['id'])

    def test__check_fs_mounted(self):
        self._driver._check_fs_mounted()

        ssh.HNASSSHBackend.check_fs_mounted.assert_called_once_with()

    def test__check_fs_mounted_not_mounted(self):
        self.mock_object(ssh.HNASSSHBackend, 'check_fs_mounted', mock.Mock(
            return_value=False))

        self.assertRaises(exception.HNASBackendException,
                          self._driver._check_fs_mounted)

        ssh.HNASSSHBackend.check_fs_mounted.assert_called_once_with()

    def test__update_share_stats(self):
        fake_data = {
            'share_backend_name': self._driver.backend_name,
            'driver_handles_share_servers':
                self._driver.driver_handles_share_servers,
            'vendor_name': 'Hitachi',
            'driver_version': '4.0.0',
            'storage_protocol': 'NFS_CIFS',
            'total_capacity_gb': 1000,
            'free_capacity_gb': 200,
            'reserved_percentage': driver.CONF.reserved_share_percentage,
            'reserved_snapshot_percentage':
                driver.CONF.reserved_share_from_snapshot_percentage,
            'qos': False,
            'thin_provisioning': True,
            'dedupe': True,
            'revert_to_snapshot_support': True,
            'mount_snapshot_support': True,
        }

        self.mock_object(ssh.HNASSSHBackend, 'get_stats', mock.Mock(
            return_value=(1000, 200, True)))
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted",
                         mock.Mock())
        self.mock_object(manila.share.driver.ShareDriver,
                         '_update_share_stats')

        self._driver._update_share_stats()

        self.assertTrue(self._driver.hnas.get_stats.called)
        (manila.share.driver.ShareDriver._update_share_stats.
         assert_called_once_with(fake_data))
        self.assertTrue(self.mock_log.info.called)

    @ddt.data(snapshot_nfs, snapshot_cifs,
              snapshot_mount_support_nfs, snapshot_mount_support_cifs)
    def test_ensure_snapshot(self, snapshot):
        result = self._driver.ensure_snapshot('context', snapshot)

        if snapshot['share'].get('mount_snapshot_support'):
            expected = [
                self._get_export(
                    snapshot['id'], snapshot['share']['share_proto'],
                    self._driver.hnas_evs_ip, False, is_snapshot=True),
                self._get_export(
                    snapshot['id'], snapshot['share']['share_proto'],
                    self._driver.hnas_admin_network_ip, True,
                    is_snapshot=True)]

            if snapshot['share']['share_proto'].lower() == 'nfs':
                ssh.HNASSSHBackend.check_export.assert_called_once_with(
                    snapshot['id'], is_snapshot=True)
                self.assertFalse(ssh.HNASSSHBackend.check_cifs.called)
            else:
                ssh.HNASSSHBackend.check_cifs.assert_called_once_with(
                    snapshot['id'])
                self.assertFalse(ssh.HNASSSHBackend.check_export.called)
        else:
            expected = None

        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snapshot['provider_location'])
        self.assertEqual(expected, result)

    def test_manage_existing_snapshot(self):
        self.mock_object(ssh.HNASSSHBackend, 'check_directory',
                         mock.Mock(return_value=True))
        self.mock_object(self._driver, '_ensure_snapshot',
                         mock.Mock(return_value=[]))

        path_info = manage_snapshot['provider_location'].split('/')
        hnas_snapshot_id = path_info[3]

        out = self._driver.manage_existing_snapshot(manage_snapshot,
                                                    {'size': 20})

        ssh.HNASSSHBackend.check_directory.assert_called_with(
            '/snapshots/aa4a7710-f326-41fb-ad18-b4ad587fc87a'
            '/snapshot18-05-2106')
        self._driver._ensure_snapshot.assert_called_with(
            manage_snapshot,
            hnas_snapshot_id)
        self.assertEqual(20, out['size'])
        self.assertTrue(self.mock_log.debug.called)
        self.assertTrue(self.mock_log.info.called)

    @ddt.data(None, exception.HNASItemNotFoundException('Fake error.'))
    def test_manage_existing_snapshot_with_mount_support(self, exc):
        export_locations = [{
            'path': '172.24.44.10:/snapshots/'
                    '3377b015-a695-4a5a-8aa5-9b931b023380'}]

        self.mock_object(ssh.HNASSSHBackend, 'check_directory',
                         mock.Mock(return_value=True))
        self.mock_object(self._driver, '_ensure_snapshot',
                         mock.Mock(return_value=[], side_effect=exc))
        self.mock_object(self._driver, '_get_export_locations',
                         mock.Mock(return_value=export_locations))
        if exc:
            self.mock_object(self._driver, '_create_export')

        path_info = snapshot_mount_support_nfs['provider_location'].split('/')
        hnas_snapshot_id = path_info[3]

        out = self._driver.manage_existing_snapshot(
            snapshot_mount_support_nfs,
            {'size': 20, 'export_locations': export_locations})

        ssh.HNASSSHBackend.check_directory.assert_called_with(
            '/snapshots/62125744-fcdd-4f55-a8c1-d1498102f634'
            '/3377b015-a695-4a5a-8aa5-9b931b023380')
        self._driver._ensure_snapshot.assert_called_with(
            snapshot_mount_support_nfs,
            hnas_snapshot_id)
        self._driver._get_export_locations.assert_called_with(
            snapshot_mount_support_nfs['share']['share_proto'],
            hnas_snapshot_id,
            is_snapshot=True)
        if exc:
            self._driver._create_export.assert_called_with(
                snapshot_mount_support_nfs['share_id'],
                snapshot_mount_support_nfs['share']['share_proto'],
                snapshot_id=hnas_snapshot_id)
        self.assertEqual(20, out['size'])
        self.assertEqual(export_locations, out['export_locations'])
        self.assertTrue(self.mock_log.debug.called)
        self.assertTrue(self.mock_log.info.called)

    @ddt.data('fake_size', '128GB', '512 GB', {'size': 128})
    def test_manage_snapshot_invalid_size_exception(self, size):
        self.assertRaises(exception.ManageInvalidShareSnapshot,
                          self._driver.manage_existing_snapshot,
                          manage_snapshot, {'size': size})

    def test_manage_snapshot_size_not_provided_exception(self):
        self.assertRaises(exception.ManageInvalidShareSnapshot,
                          self._driver.manage_existing_snapshot,
                          manage_snapshot, {})

    @ddt.data('/root/snapshot_id', '/snapshots/share1/snapshot_id',
              '/directory1', 'snapshots/share1/snapshot_id')
    def test_manage_snapshot_invalid_path_exception(self, path):
        snap_copy = manage_snapshot.copy()
        snap_copy['provider_location'] = path

        self.assertRaises(exception.ManageInvalidShareSnapshot,
                          self._driver.manage_existing_snapshot,
                          snap_copy, {'size': 20})
        self.assertTrue(self.mock_log.debug.called)

    def test_manage_inexistent_snapshot_exception(self):
        self.mock_object(ssh.HNASSSHBackend, 'check_directory',
                         mock.Mock(return_value=False))

        self.assertRaises(exception.ManageInvalidShareSnapshot,
                          self._driver.manage_existing_snapshot,
                          manage_snapshot, {'size': 20})
        self.assertTrue(self.mock_log.debug.called)

    def test_unmanage_snapshot(self):
        self._driver.unmanage_snapshot(snapshot_nfs)
        self.assertTrue(self.mock_log.info.called)

    @ddt.data({'snap': snapshot_nfs, 'exc': None},
              {'snap': snapshot_cifs, 'exc': None},
              {'snap': snapshot_nfs,
               'exc': exception.HNASNothingToCloneException('fake')},
              {'snap': snapshot_cifs,
               'exc': exception.HNASNothingToCloneException('fake')})
    @ddt.unpack
    def test_revert_to_snapshot(self, exc, snap):
        self.mock_object(driver.HitachiHNASDriver, "_check_fs_mounted")
        self.mock_object(ssh.HNASSSHBackend, 'tree_delete')
        self.mock_object(ssh.HNASSSHBackend, 'vvol_create')
        self.mock_object(ssh.HNASSSHBackend, 'quota_add')
        self.mock_object(ssh.HNASSSHBackend, 'tree_clone',
                         mock.Mock(side_effect=exc))

        self._driver.revert_to_snapshot('context', snap, None, None)

        driver.HitachiHNASDriver._check_fs_mounted.assert_called_once_with()
        ssh.HNASSSHBackend.tree_delete.assert_called_once_with(
            '/'.join(('/shares', snap['share_id'])))
        ssh.HNASSSHBackend.vvol_create.assert_called_once_with(
            snap['share_id'])
        ssh.HNASSSHBackend.quota_add.assert_called_once_with(
            snap['share_id'], 2)
        ssh.HNASSSHBackend.tree_clone.assert_called_once_with(
            '/'.join(('/snapshots', snap['share_id'], snap['id'])),
            '/'.join(('/shares', snap['share_id'])))
        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snap['provider_location'])

        if exc:
            self.assertTrue(self.mock_log.warning.called)
        self.assertTrue(self.mock_log.info.called)

    def test_nfs_snapshot_update_access_allow(self):
        access1 = {
            'access_type': 'ip',
            'access_to': '172.24.10.10',
        }
        access2 = {
            'access_type': 'ip',
            'access_to': '172.31.20.20',
        }
        access_list = [access1, access2]

        self.mock_object(ssh.HNASSSHBackend, "update_nfs_access_rule")

        self._driver.snapshot_update_access('ctxt', snapshot_nfs, access_list,
                                            access_list, [])

        ssh.HNASSSHBackend.update_nfs_access_rule.assert_called_once_with(
            [access1['access_to'] + '(ro)', access2['access_to'] + '(ro)'],
            snapshot_id=snapshot_nfs['id'])
        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snapshot_nfs['provider_location'])
        self.assertTrue(self.mock_log.debug.called)

    def test_nfs_snapshot_update_access_deny(self):
        access1 = {
            'access_type': 'ip',
            'access_to': '172.24.10.10',
        }

        self.mock_object(ssh.HNASSSHBackend, "update_nfs_access_rule")

        self._driver.snapshot_update_access('ctxt', snapshot_nfs, [],
                                            [], [access1])

        ssh.HNASSSHBackend.update_nfs_access_rule.assert_called_once_with(
            [], snapshot_id=snapshot_nfs['id'])
        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snapshot_nfs['provider_location'])
        self.assertTrue(self.mock_log.debug.called)

    def test_nfs_snapshot_update_access_invalid_access_type(self):
        access1 = {
            'access_type': 'user',
            'access_to': 'user1',
        }

        self.assertRaises(exception.InvalidSnapshotAccess,
                          self._driver.snapshot_update_access, 'ctxt',
                          snapshot_nfs, [access1], [], [])
        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snapshot_nfs['provider_location'])

    def test_cifs_snapshot_update_access_allow(self):
        access1 = {
            'access_type': 'user',
            'access_to': 'fake_user1',
        }

        self.mock_object(ssh.HNASSSHBackend, 'cifs_allow_access')

        self._driver.snapshot_update_access('ctxt', snapshot_cifs, [access1],
                                            [access1], [])

        ssh.HNASSSHBackend.cifs_allow_access.assert_called_with(
            snapshot_cifs['id'], access1['access_to'], 'ar', is_snapshot=True)
        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snapshot_cifs['provider_location'])
        self.assertTrue(self.mock_log.debug.called)

    def test_cifs_snapshot_update_access_deny(self):
        access1 = {
            'access_type': 'user',
            'access_to': 'fake_user1',
        }

        self.mock_object(ssh.HNASSSHBackend, 'cifs_deny_access')

        self._driver.snapshot_update_access('ctxt', snapshot_cifs, [], [],
                                            [access1])

        ssh.HNASSSHBackend.cifs_deny_access.assert_called_with(
            snapshot_cifs['id'], access1['access_to'], is_snapshot=True)
        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snapshot_cifs['provider_location'])
        self.assertTrue(self.mock_log.debug.called)

    def test_cifs_snapshot_update_access_recovery_mode(self):
        access1 = {
            'access_type': 'user',
            'access_to': 'fake_user1',
        }
        access2 = {
            'access_type': 'user',
            'access_to': 'HDS\\fake_user2',
        }
        access_list = [access1, access2]
        permission_list = [('fake_user1', 'ar'), ('HDS\\fake_user2', 'ar')]
        formatted_user = r'"\{1}{0}\{1}"'.format(access2['access_to'], '"')

        self.mock_object(ssh.HNASSSHBackend, 'list_cifs_permissions',
                         mock.Mock(return_value=permission_list))
        self.mock_object(ssh.HNASSSHBackend, 'cifs_deny_access')
        self.mock_object(ssh.HNASSSHBackend, 'cifs_allow_access')

        self._driver.snapshot_update_access('ctxt', snapshot_cifs, access_list,
                                            [], [])

        ssh.HNASSSHBackend.list_cifs_permissions.assert_called_once_with(
            snapshot_cifs['id'])
        ssh.HNASSSHBackend.cifs_deny_access.assert_called_with(
            snapshot_cifs['id'], formatted_user, is_snapshot=True)
        ssh.HNASSSHBackend.cifs_allow_access.assert_called_with(
            snapshot_cifs['id'], access2['access_to'].replace('\\', '\\\\'),
            'ar', is_snapshot=True)
        ssh.HNASSSHBackend.check_directory.assert_called_once_with(
            snapshot_cifs['provider_location'])
        self.assertTrue(self.mock_log.debug.called)



#    Copyright 2011 OpenStack LLC
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manila import context
from manila import test


class ContextTestCase(test.TestCase):

    def test_request_context_elevated(self):
        user_context = context.RequestContext(
            'fake_user', 'fake_project', is_admin=False)
        self.assertFalse(user_context.is_admin)
        self.assertEqual([], user_context.roles)
        admin_context = user_context.elevated()
        self.assertFalse(user_context.is_admin)
        self.assertTrue(admin_context.is_admin)
        self.assertNotIn('admin', user_context.roles)
        self.assertIn('admin', admin_context.roles)

    def test_request_context_sets_is_admin(self):
        ctxt = context.RequestContext('111',
                                      '222',
                                      roles=['admin', 'weasel'])
        self.assertTrue(ctxt.is_admin)

    def test_request_context_sets_is_admin_upcase(self):
        ctxt = context.RequestContext('111',
                                      '222',
                                      roles=['Admin', 'weasel'])
        self.assertTrue(ctxt.is_admin)

    def test_request_context_read_deleted(self):
        ctxt = context.RequestContext('111',
                                      '222',
                                      read_deleted='yes')
        self.assertEqual('yes', ctxt.read_deleted)

        ctxt.read_deleted = 'no'
        self.assertEqual('no', ctxt.read_deleted)

    def test_request_context_read_deleted_invalid(self):
        self.assertRaises(ValueError,
                          context.RequestContext,
                          '111',
                          '222',
                          read_deleted=True)

        ctxt = context.RequestContext('111', '222')
        self.assertRaises(ValueError,
                          setattr,
                          ctxt,
                          'read_deleted',
                          True)



# Copyright (c) 2015 Clinton Knight.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
NetApp cDOT NFS protocol helper class.
"""

import uuid

from oslo_log import log
import six

from manila.common import constants
from manila import exception
from manila.share.drivers.netapp.dataontap.protocols import base
from manila.share.drivers.netapp import utils as na_utils


LOG = log.getLogger(__name__)


class NetAppCmodeNFSHelper(base.NetAppBaseHelper):
    """NetApp cDOT NFS protocol helper class."""

    @staticmethod
    def _escaped_address(address):
        if ':' in address:
            return '[%s]' % address
        else:
            return address

    @na_utils.trace
    def create_share(self, share, share_name,
                     clear_current_export_policy=True,
                     ensure_share_already_exists=False, replica=False,
                     is_flexgroup=False):
        """Ensures the share export policy is set correctly.

        The export policy must have the same name as the share. If it matches,
        nothing is done. Otherwise, the possible scenarios:

        1. policy as 'default': a new export policy is created.
        2. policy as any name: renames the assigned policy to match the name.

        :param share: share entity.
        :param share_name: share name that must be the export policy name.
        :param clear_current_export_policy: set the policy to 'default' before
        the check.
        :param ensure_share_already_exists: ignored, CIFS only.
        :param replica: it is a replica volume (DP type).
        :param is_flexgroup: whether the share is a FlexGroup or not.
        """

        if clear_current_export_policy:
            self._client.clear_nfs_export_policy_for_volume(share_name)
        self._ensure_export_policy(share, share_name)

        if is_flexgroup:
            volume_info = self._client.get_volume(share_name)
            export_path = volume_info['junction-path']
        else:
            export_path = self._client.get_volume_junction_path(share_name)

        # Return a callback that may be used for generating export paths
        # for this share.
        return (lambda export_address, export_path=export_path:
                ':'.join([self._escaped_address(export_address),
                          export_path]))

    @na_utils.trace
    @base.access_rules_synchronized
    def delete_share(self, share, share_name):
        """Deletes NFS share."""
        LOG.debug('Deleting NFS export policy for share %s', share['id'])
        export_policy_name = self._get_export_policy_name(share)
        self._client.clear_nfs_export_policy_for_volume(share_name)
        self._client.soft_delete_nfs_export_policy(export_policy_name)

    @na_utils.trace
    @base.access_rules_synchronized
    def update_access(self, share, share_name, rules):
        """Replaces the list of access rules known to the backend storage."""

        # Ensure rules are valid
        for rule in rules:
            self._validate_access_rule(rule)

        # Sort rules by ascending network size
        new_rules = {rule['access_to']: rule['access_level'] for rule in rules}
        addresses = sorted(new_rules, reverse=True)

        # Ensure current export policy has the name we expect
        self._ensure_export_policy(share, share_name)
        export_policy_name = self._get_export_policy_name(share)

        # Make temp policy names so this non-atomic workflow remains resilient
        # across process interruptions.
        temp_new_export_policy_name = self._get_temp_export_policy_name()
        temp_old_export_policy_name = self._get_temp_export_policy_name()

        # Create new export policy
        self._client.create_nfs_export_policy(temp_new_export_policy_name)

        # Get authentication methods, based on Vserver configuration
        auth_methods = self._get_auth_methods()

        # Add new rules to new policy
        for address in addresses:
            self._client.add_nfs_export_rule(
                temp_new_export_policy_name, address,
                self._is_readonly(new_rules[address]), auth_methods)

        # Rename policy currently in force
        LOG.info('Renaming NFS export policy for share %(share)s to '
                 '%(policy)s.',
                 {'share': share_name, 'policy': temp_old_export_policy_name})
        self._client.rename_nfs_export_policy(export_policy_name,
                                              temp_old_export_policy_name)

        # Switch share to the new policy
        LOG.info('Setting NFS export policy for share %(share)s to '
                 '%(policy)s.',
                 {'share': share_name, 'policy': temp_new_export_policy_name})
        self._client.set_nfs_export_policy_for_volume(
            share_name, temp_new_export_policy_name)

        # Delete old policy
        self._client.soft_delete_nfs_export_policy(temp_old_export_policy_name)

        # Rename new policy to its final name
        LOG.info('Renaming NFS export policy for share %(share)s to '
                 '%(policy)s.',
                 {'share': share_name, 'policy': export_policy_name})
        self._client.rename_nfs_export_policy(temp_new_export_policy_name,
                                              export_policy_name)

    @na_utils.trace
    def _validate_access_rule(self, rule):
        """Checks whether access rule type and level are valid."""

        if rule['access_type'] != 'ip':
            msg = ("Clustered Data ONTAP supports only 'ip' type for share "
                   "access rules with NFS protocol.")
            raise exception.InvalidShareAccess(reason=msg)

        if rule['access_level'] not in constants.ACCESS_LEVELS:
            raise exception.InvalidShareAccessLevel(level=rule['access_level'])

    @na_utils.trace
    def get_target(self, share):
        """Returns ID of target ONTAP device based on export location."""
        return self._get_export_location(share)[0]

    @na_utils.trace
    def get_share_name_for_share(self, share):
        """Returns the flexvol name that hosts a share."""
        _, volume_junction_path = self._get_export_location(share)
        volume = self._client.get_volume_at_junction_path(volume_junction_path)
        return volume.get('name') if volume else None

    @na_utils.trace
    def _get_export_location(self, share):
        """Returns IP address and export location of an NFS share."""
        export_location = self._get_share_export_location(share) or ':'
        result = export_location.rsplit(':', 1)
        if len(result) != 2:
            return ['', '']
        return result

    @staticmethod
    def _get_temp_export_policy_name():
        """Builds export policy name for an NFS share."""
        return 'temp_' + six.text_type(uuid.uuid1()).replace('-', '_')

    @staticmethod
    def _get_export_policy_name(share):
        """Builds export policy name for an NFS share."""
        return 'policy_' + share['id'].replace('-', '_')

    @na_utils.trace
    def _ensure_export_policy(self, share, share_name):
        """Ensures a flexvol/share has an export policy.

        This method ensures a flexvol has an export policy with a name
        containing the share ID.  For legacy reasons, this may not
        always be the case.
        """
        expected_export_policy = self._get_export_policy_name(share)
        actual_export_policy = self._client.get_nfs_export_policy_for_volume(
            share_name)

        if actual_export_policy == expected_export_policy:
            return
        elif actual_export_policy == 'default':
            self._client.create_nfs_export_policy(expected_export_policy)
            self._client.set_nfs_export_policy_for_volume(
                share_name, expected_export_policy)
        else:
            self._client.rename_nfs_export_policy(actual_export_policy,
                                                  expected_export_policy)

    @na_utils.trace
    def _get_auth_methods(self):
        """Returns authentication methods for export policy rules.

        This method returns the authentication methods to be configure in an
        export policy rule, based on security services configuration set in
        the current Vserver. If Kerberos is enabled in vServer LIFs, the auth
        methods will be configure to support 'krb5', 'krb5i' and 'krb5p'. The
        default authentication method is 'sys' (AUTH_SYS).
        """
        kerberos_enabled = self._client.is_kerberos_enabled()
        return ['krb5', 'krb5i', 'krb5p'] if kerberos_enabled else ['sys']

    @na_utils.trace
    def cleanup_demoted_replica(self, share, share_name):
        """Cleans up export NFS policy for a demoted replica."""
        self.delete_share(share, share_name)
        return



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manila import exception
from manila.i18n import _


class Resource(object):

    SHARE = 'SHARE'
    SHARE_GROUP = 'SHARE_GROUP'
    SHARE_REPLICA = 'SHARE_REPLICA'
    SHARE_SNAPSHOT = 'SHARE_SNAPSHOT'
    SECURITY_SERVICE = 'SECURITY_SERVICE'


class Action(object):

    ALLOCATE_HOST = ('001', _('allocate host'))
    CREATE = ('002', _('create'))
    DELETE_ACCESS_RULES = ('003', _('delete access rules'))
    PROMOTE = ('004', _('promote'))
    UPDATE = ('005', _('update'))
    REVERT_TO_SNAPSHOT = ('006', _('revert to snapshot'))
    DELETE = ('007', _('delete'))
    EXTEND = ('008', _('extend'))
    SHRINK = ('009', _('shrink'))
    UPDATE_ACCESS_RULES = ('010', _('update access rules'))
    ADD_UPDATE_SECURITY_SERVICE = ('011', _('add or update security service'))
    ALL = (
        ALLOCATE_HOST,
        CREATE,
        DELETE_ACCESS_RULES,
        PROMOTE,
        UPDATE,
        REVERT_TO_SNAPSHOT,
        DELETE,
        EXTEND,
        SHRINK,
        UPDATE_ACCESS_RULES,
        ADD_UPDATE_SECURITY_SERVICE,
    )


class Detail(object):

    UNKNOWN_ERROR = ('001', _('An unknown error occurred.'))
    NO_VALID_HOST = (
        '002', _("No storage could be allocated for this share request. "
                 "Trying again with a different size or share type may "
                 "succeed."))
    UNEXPECTED_NETWORK = (
        '003', _("Driver does not expect share-network to be provided with "
                 "current configuration."))
    NO_SHARE_SERVER = (
        '004', _("Could not find an existing share server or allocate one on "
                 "the share network provided. You may use a different share "
                 "network, or verify the network details in the share network "
                 "and retry your request. If this doesn't work, contact your "
                 "administrator to troubleshoot issues with your network."))
    NO_ACTIVE_AVAILABLE_REPLICA = (
        '005', _("An 'active' replica must exist in 'available' state to "
                 "create a new replica for share."))
    NO_ACTIVE_REPLICA = (
        '006', _("Share has no replica with 'replica_state' set to 'active'."))

    FILTER_MSG = _("No storage could be allocated for this share request, "
                   "%s filter didn't succeed.")
    FILTER_AVAILABILITY = ('007', FILTER_MSG % 'AvailabilityZone')
    FILTER_CAPABILITIES = ('008', FILTER_MSG % 'Capabilities')
    FILTER_CAPACITY = ('009', FILTER_MSG % 'Capacity')
    FILTER_DRIVER = ('010', FILTER_MSG % 'Driver')
    FILTER_IGNORE = ('011', FILTER_MSG % 'IgnoreAttemptedHosts')
    FILTER_JSON = ('012', FILTER_MSG % 'Json')
    FILTER_RETRY = ('013', FILTER_MSG % 'Retry')
    FILTER_REPLICATION = ('014', FILTER_MSG % 'ShareReplication')
    DRIVER_FAILED_EXTEND = (
        '015',
        _("Share Driver failed to extend share, The share status has been "
          "set to extending_error. This action cannot be re-attempted until "
          "the status has been rectified. Contact your administrator to "
          "determine the cause of this failure."))
    FILTER_CREATE_FROM_SNAPSHOT = ('016', FILTER_MSG % 'CreateFromSnapshot')
    DRIVER_FAILED_CREATING_FROM_SNAP = (
        '017',
        _("Share Driver has failed to create the share from snapshot. This "
          "operation can be re-attempted by creating a new share. Contact "
          "your administrator to determine the cause of this failure."))
    DRIVER_REFUSED_SHRINK = (
        '018',
        _("Share Driver refused to shrink the share. The size to be shrunk is"
          " smaller than the current used space. The share status has been"
          " set to available. Please select a size greater than the current"
          " used space."))
    DRIVER_FAILED_SHRINK = (
        '019',
        _("Share Driver does not support shrinking shares."
          " Shrinking share operation failed."))
    FORBIDDEN_CLIENT_ACCESS = (
        '020',
        _("Failed to grant access to client. The client ID used may be "
          "forbidden. You may try again with a different client identifier."))
    UNSUPPORTED_CLIENT_ACCESS = (
        '021',
        _("Failed to grant access to client. The access level or type may "
          "be unsupported. You may try again with a different access level "
          "or access type."))
    UNSUPPORTED_ADD_UDPATE_SECURITY_SERVICE = (
        '022',
        _("Share driver has failed to setup one or more security services "
          "that are associated with the used share network. The security "
          "service may be unsupported or the provided parameters are invalid. "
          "You may try again with a different set of configurations."))
    SECURITY_SERVICE_FAILED_AUTH = (
        '023',
        _("Share Driver failed to create share due to a security service "
          "authentication issue. The security service user has either "
          "insufficient privileges or wrong credentials. Please check your "
          "user, password, ou and domain."))

    NO_DEFAULT_SHARE_TYPE = (
        '024',
        _("No default share type has been made available. "
          "You must specify a share type for creating shares."))
    MISSING_SECURITY_SERVICE = (
        '025',
        _("Share Driver failed to create share because a security service "
          "has not been added to the share network used. Please add a "
          "security service to the share network."))

    ALL = (
        UNKNOWN_ERROR,
        NO_VALID_HOST,
        UNEXPECTED_NETWORK,
        NO_SHARE_SERVER,
        NO_ACTIVE_AVAILABLE_REPLICA,
        NO_ACTIVE_REPLICA,
        FILTER_AVAILABILITY,
        FILTER_CAPABILITIES,
        FILTER_CAPACITY,
        FILTER_DRIVER,
        FILTER_IGNORE,
        FILTER_JSON,
        FILTER_RETRY,
        FILTER_REPLICATION,
        DRIVER_FAILED_EXTEND,
        FILTER_CREATE_FROM_SNAPSHOT,
        DRIVER_FAILED_CREATING_FROM_SNAP,
        DRIVER_REFUSED_SHRINK,
        DRIVER_FAILED_SHRINK,
        FORBIDDEN_CLIENT_ACCESS,
        UNSUPPORTED_CLIENT_ACCESS,
        UNSUPPORTED_ADD_UDPATE_SECURITY_SERVICE,
        SECURITY_SERVICE_FAILED_AUTH,
        NO_DEFAULT_SHARE_TYPE,
        MISSING_SECURITY_SERVICE,
    )

    # Exception and detail mappings
    EXCEPTION_DETAIL_MAPPINGS = {
        NO_VALID_HOST: ['NoValidHost'],
    }

    # Use special code for each filter rather then categorize all as
    # NO_VALID_HOST
    FILTER_DETAIL_MAPPINGS = {
        'AvailabilityZoneFilter': FILTER_AVAILABILITY,
        'CapabilitiesFilter': FILTER_CAPABILITIES,
        'CapacityFilter': FILTER_CAPACITY,
        'DriverFilter': FILTER_DRIVER,
        'IgnoreAttemptedHostsFilter': FILTER_IGNORE,
        'JsonFilter': FILTER_JSON,
        'RetryFilter': FILTER_RETRY,
        'ShareReplicationFilter': FILTER_REPLICATION,
        'CreateFromSnapshotFilter': FILTER_CREATE_FROM_SNAPSHOT,
    }


def translate_action(action_id):
    action_message = next((action[1] for action in Action.ALL
                           if action[0] == action_id), None)
    return action_message or 'unknown action'


def translate_detail(detail_id):
    detail_message = next((action[1] for action in Detail.ALL
                           if action[0] == detail_id), None)
    return detail_message or Detail.UNKNOWN_ERROR[1]


def translate_detail_id(excep, detail):
    if excep is not None:
        detail = _translate_exception_to_detail(excep)
    if detail in Detail.ALL:
        return detail[0]
    return Detail.UNKNOWN_ERROR[0]


def _translate_exception_to_detail(ex):
    if isinstance(ex, exception.NoValidHost):
        # if NoValidHost was raised because a filter failed (a filter
        # didn't return any hosts), use a filter-specific detail
        details = getattr(ex, 'detail_data', {})
        last_filter = details.get('last_filter')
        return Detail.FILTER_DETAIL_MAPPINGS.get(
            last_filter, Detail.NO_VALID_HOST)
    else:
        for key, value in Detail.EXCEPTION_DETAIL_MAPPINGS.items():
            if ex.__class__.__name__ in value:
                return key
